from collections import OrderedDict
from typing import Optional

from ..en import Provider as AddressProvider


class Provider(AddressProvider):
    city_prefixes = ("North", "East", "West", "South", "New", "Lake", "Port")

    city_suffixes = (
        "town",
        "ton",
        "land",
        "ville",
        "berg",
        "burgh",
        "borough",
        "bury",
        "view",
        "port",
        "mouth",
        "stad",
        "furt",
        "chester",
        "mouth",
        "fort",
        "haven",
        "side",
        "shire",
    )

    building_number_formats = ("#####", "####", "###")

    street_suffixes = (
        "Alley",
        "Avenue",
        "Branch",
        "Bridge",
        "Brook",
        "Brooks",
        "Burg",
        "Burgs",
        "Bypass",
        "Camp",
        "Canyon",
        "Cape",
        "Causeway",
        "Center",
        "Centers",
        "Circle",
        "Circles",
        "Cliff",
        "Cliffs",
        "Club",
        "Common",
        "Corner",
        "Corners",
        "Course",
        "Court",
        "Courts",
        "Cove",
        "Coves",
        "Creek",
        "Crescent",
        "Crest",
        "Crossing",
        "Crossroad",
        "Curve",
        "Dale",
        "Dam",
        "Divide",
        "Drive",
        "Drive",
        "Drives",
        "Estate",
        "Estates",
        "Expressway",
        "Extension",
        "Extensions",
        "Fall",
        "Falls",
        "Ferry",
        "Field",
        "Fields",
        "Flat",
        "Flats",
        "Ford",
        "Fords",
        "Forest",
        "Forge",
        "Forges",
        "Fork",
        "Forks",
        "Fort",
        "Freeway",
        "Garden",
        "Gardens",
        "Gateway",
        "Glen",
        "Glens",
        "Green",
        "Greens",
        "Grove",
        "Groves",
        "Harbor",
        "Harbors",
        "Haven",
        "Heights",
        "Highway",
        "Hill",
        "Hills",
        "Hollow",
        "Inlet",
        "Inlet",
        "Island",
        "Island",
        "Islands",
        "Islands",
        "Isle",
        "Isle",
        "Junction",
        "Junctions",
        "Key",
        "Keys",
        "Knoll",
        "Knolls",
        "Lake",
        "Lakes",
        "Land",
        "Landing",
        "Lane",
        "Light",
        "Lights",
        "Loaf",
        "Lock",
        "Locks",
        "Locks",
        "Lodge",
        "Lodge",
        "Loop",
        "Mall",
        "Manor",
        "Manors",
        "Meadow",
        "Meadows",
        "Mews",
        "Mill",
        "Mills",
        "Mission",
        "Mission",
        "Motorway",
        "Mount",
        "Mountain",
        "Mountain",
        "Mountains",
        "Mountains",
        "Neck",
        "Orchard",
        "Oval",
        "Overpass",
        "Park",
        "Parks",
        "Parkway",
        "Parkways",
        "Pass",
        "Passage",
        "Path",
        "Pike",
        "Pine",
        "Pines",
        "Place",
        "Plain",
        "Plains",
        "Plains",
        "Plaza",
        "Plaza",
        "Point",
        "Points",
        "Port",
        "Port",
        "Ports",
        "Ports",
        "Prairie",
        "Prairie",
        "Radial",
        "Ramp",
        "Ranch",
        "Rapid",
        "Rapids",
        "Rest",
        "Ridge",
        "Ridges",
        "River",
        "Road",
        "Road",
        "Roads",
        "Roads",
        "Route",
        "Row",
        "Rue",
        "Run",
        "Shoal",
        "Shoals",
        "Shore",
        "Shores",
        "Skyway",
        "Spring",
        "Springs",
        "Springs",
        "Spur",
        "Spurs",
        "Square",
        "Square",
        "Squares",
        "Squares",
        "Station",
        "Station",
        "Stravenue",
        "Stravenue",
        "Stream",
        "Stream",
        "Street",
        "Street",
        "Streets",
        "Summit",
        "Summit",
        "Terrace",
        "Throughway",
        "Trace",
        "Track",
        "Trafficway",
        "Trail",
        "Trail",
        "Tunnel",
        "Tunnel",
        "Turnpike",
        "Turnpike",
        "Underpass",
        "Union",
        "Unions",
        "Valley",
        "Valleys",
        "Via",
        "Viaduct",
        "View",
        "Views",
        "Village",
        "Village",
        "Villages",
        "Ville",
        "Vista",
        "Vista",
        "Walk",
        "Walks",
        "Wall",
        "Way",
        "Ways",
        "Well",
        "Wells",
    )

    postcode_formats = ("#####", "#####-####")

    states = (
        "Alabama",
        "Alaska",
        "Arizona",
        "Arkansas",
        "California",
        "Colorado",
        "Connecticut",
        "Delaware",
        "Florida",
        "Georgia",
        "Hawaii",
        "Idaho",
        "Illinois",
        "Indiana",
        "Iowa",
        "Kansas",
        "Kentucky",
        "Louisiana",
        "Maine",
        "Maryland",
        "Massachusetts",
        "Michigan",
        "Minnesota",
        "Mississippi",
        "Missouri",
        "Montana",
        "Nebraska",
        "Nevada",
        "New Hampshire",
        "New Jersey",
        "New Mexico",
        "New York",
        "North Carolina",
        "North Dakota",
        "Ohio",
        "Oklahoma",
        "Oregon",
        "Pennsylvania",
        "Rhode Island",
        "South Carolina",
        "South Dakota",
        "Tennessee",
        "Texas",
        "Utah",
        "Vermont",
        "Virginia",
        "Washington",
        "West Virginia",
        "Wisconsin",
        "Wyoming",
    )
    states_abbr = (
        "AL",
        "AK",
        "AZ",
        "AR",
        "CA",
        "CO",
        "CT",
        "DE",
        "DC",
        "FL",
        "GA",
        "HI",
        "ID",
        "IL",
        "IN",
        "IA",
        "KS",
        "KY",
        "LA",
        "ME",
        "MD",
        "MA",
        "MI",
        "MN",
        "MS",
        "MO",
        "MT",
        "NE",
        "NV",
        "NH",
        "NJ",
        "NM",
        "NY",
        "NC",
        "ND",
        "OH",
        "OK",
        "OR",
        "PA",
        "RI",
        "SC",
        "SD",
        "TN",
        "TX",
        "UT",
        "VT",
        "VA",
        "WA",
        "WV",
        "WI",
        "WY",
    )

    states_postcode = {
        "AL": (35004, 36925),
        "AK": (99501, 99950),
        "AZ": (85001, 86556),
        "AR": (71601, 72959),
        "CA": (90001, 96162),
        "CO": (80001, 81658),
        "CT": (6001, 6389),
        "DE": (19701, 19980),
        "DC": (20001, 20039),
        "FL": (32004, 34997),
        "GA": (30001, 31999),
        "HI": (96701, 96898),
        "ID": (83201, 83876),
        "IL": (60001, 62999),
        "IN": (46001, 47997),
        "IA": (50001, 52809),
        "KS": (66002, 67954),
        "KY": (40003, 42788),
        "LA": (70001, 71232),
        "ME": (3901, 4992),
        "MD": (20812, 21930),
        "MA": (1001, 2791),
        "MI": (48001, 49971),
        "MN": (55001, 56763),
        "MS": (38601, 39776),
        "MO": (63001, 65899),
        "MT": (59001, 59937),
        "NE": (68001, 68118),
        "NV": (88901, 89883),
        "NH": (3031, 3897),
        "NJ": (7001, 8989),
        "NM": (87001, 88441),
        "NY": (10001, 14905),
        "NC": (27006, 28909),
        "ND": (58001, 58856),
        "OH": (43001, 45999),
        "OK": (73001, 73199),
        "OR": (97001, 97920),
        "PA": (15001, 19640),
        "RI": (2801, 2940),
        "SC": (29001, 29948),
        "SD": (57001, 57799),
        "TN": (37010, 38589),
        "TX": (75503, 79999),
        "UT": (84001, 84784),
        "VT": (5001, 5495),
        "VA": (22001, 24658),
        "WA": (98001, 99403),
        "WV": (24701, 26886),
        "WI": (53001, 54990),
        "WY": (82001, 83128),
    }

    territories_abbr = (
        "AS",
        "FM",
        "GU",
        "MH",
        "MP",
        "PW",
        "PR",
        "VI",
    )

    states_and_territories_abbr = states_abbr + territories_abbr

    military_state_abbr = ("AE", "AA", "AP")

    military_ship_prefix = ("USS", "USNS", "USNV", "USCGC")

    military_apo_format = "PSC ####, Box ####"

    military_dpo_format = "Unit #### Box ####"

    city_formats = (
        "{{city_prefix}} {{first_name}}{{city_suffix}}",
        "{{city_prefix}} {{first_name}}",
        "{{first_name}}{{city_suffix}}",
        "{{last_name}}{{city_suffix}}",
    )

    street_name_formats = (
        "{{first_name}} {{street_suffix}}",
        "{{last_name}} {{street_suffix}}",
    )

    street_address_formats = (
        "{{building_number}} {{street_name}}",
        "{{building_number}} {{street_name}} {{secondary_address}}",
    )

    address_formats = OrderedDict(
        (
            ("{{street_address}}\n{{city}}, {{state_abbr}} {{postcode}}", 25.0),
            #  military address formatting.
            ("{{military_apo}}\nAPO {{military_state}} {{postcode}}", 1.0),
            (
                "{{military_ship}} {{last_name}}\nFPO {{military_state}} {{postcode}}",
                1.0,
            ),
            ("{{military_dpo}}\nDPO {{military_state}} {{postcode}}", 1.0),
        )
    )

    secondary_address_formats = ("Apt. ###", "Suite ###")

    def city_prefix(self) -> str:
        return self.random_element(self.city_prefixes)

    def secondary_address(self) -> str:
        return self.numerify(self.random_element(self.secondary_address_formats))

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit

    def state_abbr(self, include_territories: bool = True) -> str:
        """
        :returns: A random state or territory abbreviation.

        :param include_territories: If True, territories will be included.
            If False, only states will be returned.
        """
        if include_territories:
            self.random_element(self.states_and_territories_abbr)
        return self.random_element(self.states_abbr)

    def postcode(self) -> str:
        return "%05d" % self.generator.random.randint(501, 99950)

    def zipcode_plus4(self) -> str:
        return "%s-%04d" % (self.zipcode(), self.generator.random.randint(1, 9999))

    def postcode_in_state(self, state_abbr: Optional[str] = None) -> str:
        """
        :returns: A random postcode within the provided state abbreviation

        :param state_abbr: A state abbreviation
        """
        if state_abbr is None:
            state_abbr = self.random_element(self.states_abbr)

        if state_abbr in self.states_abbr:
            postcode = "%d" % (
                self.generator.random.randint(
                    self.states_postcode[state_abbr][0],
                    self.states_postcode[state_abbr][1],
                )
            )

            if len(postcode) == 4:
                postcode = "0%s" % postcode

            return postcode

        else:
            raise Exception("State Abbreviation not found in list")

    def military_ship(self) -> str:
        """
        :example 'USS'
        """
        return self.random_element(self.military_ship_prefix)

    def military_state(self) -> str:
        """
        :example 'APO'
        """
        return self.random_element(self.military_state_abbr)

    def military_apo(self) -> str:
        """
        :example 'PSC 5394 Box 3492
        """
        return self.numerify(self.military_apo_format)

    def military_dpo(self) -> str:
        """
        :example 'Unit 3333 Box 9342'
        """
        return self.numerify(self.military_dpo_format)

    # Aliases
    def zipcode(self) -> str:
        return self.postcode()

    def zipcode_in_state(self, state_abbr: Optional[str] = None) -> str:
        return self.postcode_in_state(state_abbr)

    def postalcode(self) -> str:
        return self.postcode()

    def postalcode_in_state(self, state_abbr: Optional[str] = None) -> str:
        return self.postcode_in_state(state_abbr)

    def postalcode_plus4(self) -> str:
        return self.zipcode_plus4()



from .. import Provider as CurrencyProvider

# Names taken from https://std.moc.go.th/std/codelist_detail/40


class Provider(CurrencyProvider):
    # Format: (code, name)
    currencies = (
        ("AED", "ดีแรห์ม สหรัฐอาหรับเอมิเรตส์"),
        ("AFN", "อัฟกานิ"),
        ("ALL", "เลค"),
        ("AMD", "ดีแรห์ม อาร์เมเนีย"),
        ("ANG", "กิลเดอร์ เนเธอร์แลนด์แอนทิลลิส"),
        ("AOA", "ควันซา"),
        ("ARS", "เปโซ อาร์เจนตินา"),
        ("AUD", "ดอลลาร์ ออสเตรเลีย"),
        ("AWG", "กิลเดอร์ อารูบา"),
        ("AZN", "มานาท อาเซอร์ไบจาน"),
        ("BAM", "มาร์ค บอสเนียและเฮอร์เซโกวีนา"),
        ("BBD", "ดอลลาร์ บาร์เบโดส"),
        ("BDT", "ตากา"),
        ("BGN", "เลฟ บัลแกเรีย"),
        ("BHD", "ดีนาร์ บาห์เรน"),
        ("BIF", "ฟรังก์ บุรุนดี"),
        ("BMD", "ดอลลาร์ เบอร์มิวดา"),
        ("BND", "ดอลลาร์ บรูไนดารุสซาลาม"),
        ("BOB", "โบลีเวียโน"),
        ("BRL", "เรียล บราซิล"),
        ("BSD", "ดอลลาร์ บาฮามาส"),
        ("BTN", "เองกัลทรัม"),
        ("BWP", "พูลา"),
        ("BYR", "รูเบิล เบลารุส"),
        ("BZD", "ดอลลาร์ เบลีซ"),
        ("CAD", "ดอลลาร์ แคนาดา"),
        ("CDF", "ฟรังก์ คองโก"),
        ("CHF", "ฟรังก์ สวิส"),
        ("CLF", "ฟันด์ โค้ด ยูนิแดด ดี โฟเมนโต"),
        ("CLP", "เปโซ ชิลี"),
        ("CNY", "หยวนเหรินหมินปี้"),
        ("COP", "เปโซ โคลอมเบีย"),
        ("COU", "ยูนิแดด ดี วาโล เรียล"),
        ("CRC", "โคโลน คอสตาริกา"),
        ("CUC", "แปลงสภาพเปโซ"),
        ("CUP", "เปโซ คิวบา"),
        ("CVE", "เอสคูโด เคปเวิร์ด"),
        ("CZK", "คราวน์ เช็ก"),
        ("DJF", "ฟรังก์ จิบูตี"),
        ("DKK", "โครน เดนมาร์ก"),
        ("DOP", "เปโซ สาธารณรัฐโดมินิกัน"),
        ("DZD", "ดีนาร์ แอลจีเรีย"),
        ("EGP", "ปอนด์ อียิปต์"),
        ("ERN", "นาคฟา"),
        ("ETB", "เปอร์ เอธิโอเปีย"),
        ("EUR", "ยูโร"),
        ("FJD", "ดอลลาร์ ฟิจิ"),
        ("FKP", "ปอนด์ หมู่เกาะฟอล์กแลนด์"),
        ("GBP", "ปอนด์สเตอลิง"),
        ("GEL", "ลารี"),
        ("GHS", "เซดี กานา"),
        ("GIP", "ปอนด์ ยิบรอลตาร์"),
        ("GMD", "ดาราซี"),
        ("GNF", "ฟรังก์ กินี"),
        ("GTQ", "เก็ตซาล"),
        ("GYD", "ดอลลาร์ กายอานา"),
        ("HKD", "ดอลลาร์ ฮ่องกง"),
        ("HNL", "เลมพีรา ฮอนดูรัส"),
        ("HRK", "คูนา"),
        ("HTG", "กอร์ด"),
        ("HUF", "ฟอรินท์"),
        ("IDR", "รูเปีย"),
        ("ILS", "เชคเกิล อิสราเอล"),
        ("INR", "รูปี อินเดีย"),
        ("IQD", "ดีนาร์ อิรัก"),
        ("IRR", "เรียล อิหร่าน"),
        ("ISK", "โครนา ไอซ์แลนด์"),
        ("JMD", "ดอลลาร์ จาเมกา"),
        ("JOD", "ดอลลาร์ จอร์แดน"),
        ("JPY", "เยน"),
        ("KES", "ชิลลิง เคนยา"),
        ("KGS", "ซอม"),
        ("KHR", "เรียล กัมพูชา"),
        ("KMF", "ฟรังก์ คอโมโรส"),
        ("KPW", "วอน เกาหลีเหนือ"),
        ("KRW", "วอน เกาหลีใต้"),
        ("KWD", "ดีนาร์ คูเวต"),
        ("KYD", "ดอลลาร์ หมู่เกาะเคย์แมน"),
        ("KZT", "เทงเก"),
        ("LAK", "กีบ"),
        ("LBP", "ปอนด์ เลบานอน"),
        ("LKR", "รูปี ศรีลังกา"),
        ("LRD", "ดอลลาร์ ไลบีเรีย"),
        ("LSL", "โลตี"),
        ("LTL", "ลีทาส ลิทัวเนีย"),
        ("LVL", "ลัตส์ ลัตเวีย"),
        ("LYD", "ดีนาร์ ลิเบีย"),
        ("MAD", "ดีแรห์ม โมร็อกโก"),
        ("MDL", "ลิว มอลโดวา"),
        ("MGA", "อเรียรี่ มาดากัสการ์"),
        ("MKD", "ดีนาร์ มาซิโดเนีย"),
        ("MMK", "จัต"),
        ("MNT", "ทูกริค"),
        ("MOP", "พาทากา"),
        ("MRO", "อูกุยยา / อูกียา"),
        ("MUR", "รูปี มอริเชียส"),
        ("MVR", "รูฟียา"),
        ("MWK", "ควาซา มาลาวี"),
        ("MXN", "เปโซ เม็กซิโก"),
        ("MYR", "ริงกิต มาเลเซีย"),
        ("MZN", "เมททิคัล โมซัมบิก"),
        ("NAD", "ดอลลาร์ นามิเบีย"),
        ("NGN", "ไนรา"),
        ("NIO", "คอร์โดบา"),
        ("NOK", "โครน นอร์เวย์"),
        ("NPR", "รูปี เนปาล"),
        ("NZD", "ดอลลาร์ นิวซีแลนด์"),
        ("OMR", "เรียล โอมาน"),
        ("PAB", "บัลโบอา"),
        ("PEN", "ซัล เปรู"),
        ("PGK", "คีนา"),
        ("PHP", "เปโซ ฟิลิปปินส์"),
        ("PKR", "รูปี ปากีสถาน"),
        ("PLN", "สล็อตตี"),
        ("PYG", "กวารานี"),
        ("QAR", "เรียล กาตาร์"),
        ("RON", "ลิว โรมาเนีย"),
        ("RSD", "ดีนาร์ เซอร์เบีย"),
        ("RUB", "รูเบิล รัสเซีย"),
        ("RWF", "ฟรังก์ รวันดา"),
        ("SAR", "ริยัล ซาอุดีอาระเบีย"),
        ("SBD", "ดอลลาร์ หมู่เกาะโซโลมอน"),
        ("SCR", "รูปี เซเชลส์"),
        ("SDG", "ปอนด์ ซูดาน"),
        ("SEK", "โครนา สวีเดน"),
        ("SGD", "ดอลลาร์ สิงคโปร์"),
        ("SHP", "ปอนด์ เซนต์เฮเลนา"),
        ("SLL", "ลีโอน"),
        ("SOS", "ชิลลิง โซมาเลีย"),
        ("SRD", "ดอลลาร์ ซูรินาเม"),
        ("SSP", "ปอนด์ เซาท์ซูดาน"),
        ("STD", "โดบรา"),
        ("SVC", "โคโลน เอลซัลวาดอร์"),
        ("SYP", "ปอนด์ ซีเรีย"),
        ("SZL", "ลิลอนเกนี"),
        ("THB", "บาท"),
        ("TJS", "โซโมนิ"),
        ("TMT", "มานาท เติร์กเมนิสถานใหม่"),
        ("TND", "ดีนาร์ ตูนิเซีย"),
        ("TOP", "พาอานกา"),
        ("TRY", "ลีร์ ตุรกี"),
        ("TTD", "ดอลลาร์ ตรินิแดดและโตเบโก"),
        ("TWD", "ดอลลาร์ ไต้หวัน"),
        ("TZS", "ชิลลิง แทนซาเนีย"),
        ("UAH", "รีฟเนีย"),
        ("UGX", "ชิลลิง ยูกันดา"),
        ("USD", "ดอลลาร์ สหรัฐอเมริกา"),
        ("USN", "ดอลลาร์ สหรัฐอเมริกา เน็กซ์เดย์ฟัน"),
        ("UYI", "อุรุกวัย เปโซ เอ็น อุนดิดาเดซ อินเด็กซาดาซ"),
        ("UYU", "เปโซ อุรุกวัย"),
        ("UZS", "โซม อุซเบกิสถาน"),
        ("VEF", "โบลิวาร์"),
        ("VND", "ดอง"),
        ("VUV", "วาตู"),
        ("WST", "ทาลา"),
        ("XAF", "ฟรังก์ ซีเอฟเอ บีอีเอซี"),
        ("XAG", "เงิน"),
        ("XAU", "ทองคำ"),
        ("XBA", "อียูอาร์ซีโอ"),
        ("XBB", "อีเอ็มยู 6"),
        ("XBC", "บัญชี อียู 9"),
        ("XBD", "บัญชี อียู 17"),
        ("XCD", "ดอลลาร์ คาริบเบียลตะวันออก"),
        ("XDR", "สิทธิพิเศษถอนเงิน (กองทุนการเงินระหว่างประเทศ)"),
        ("XEU", "อี ซี ยู"),
        ("XFU", "ยูไอซี ฟรังก์"),
        ("XOF", "ฟรังก์ ซีเอฟเอ บีซีอีเอโอ"),
        ("XPD", "พัลเลเดียม"),
        ("XPF", "ฟรังก์ ซีเอฟพี"),
        ("XPT", "แพลตตินัม"),
        ("XSU", "ซูเคร"),
        ("XUA", "เอดีบี"),
        ("XXX", "ธุรกรรมที่ไม่มีเงินสกุลใดเกี่ยวข้อง"),
        ("YER", "เรียล เยเมน"),
        ("ZAR", "แรนด์"),
        ("ZMW", "ควาซา แซมเบีย"),
        ("ZWL", "ดอลลาร์ ซิมบับเว"),
    )



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    # Source: https://learnenglish100.com/grammar/career-job/
    jobs = (
        "محاسب",
        "ممثل",
        "ممثلة",
        "محام",
        "مضيفة في الطائرة",
        "مضيفة جوية",
        "عالم",
        "عالم اثار",
        "فنان",
        "مؤلف",
        "خباز",
        "راقصة باليه",
        "مدير بنك",
        "حلاق",
        "أحيائي",
        "حداد",
        "مجلد كتب",
        "صاحب مكتبة",
        "نباتي",
        "عامل بناء",
        "جزار",
        "مشتري",
        "نجار",
        "رئيس طهاه",
        "صيدلي",
        "قائد شرطة",
        "موظف حكومي",
        "كاتب",
        "ملحن",
        "طباخ",
        "مصحح قانوني",
        "راقصة",
        "مصمم ديكور",
        "موصل طلبيات",
        "طبيب أسنان",
        "مصصم",
        "رجل مباحث",
        "مدير او مخرج",
        "طبيب",
        "رسام",
        "خياطة",
        "سائق",
        "صباغ",
        "خبير اقتصادي",
        "محرر",
        "محرر جريدة",
        "مھندس",
        "خبير في التراث الشعبي",
        "منسق ازياء",
        "اطفائي",
        "بائع زهور",
        "جيولوجي",
        "صائغ",
        "مصمم جرافيك",
        "بائع خضار وفاكهة",
        "حلاق",
        "مصفف شعر",
        "مصور",
        "وسيط تأمين",
        "مترجم",
        "مكوى",
        "بواب",
        "جوھري",
        "صحافي",
        "قاضي",
        "غسالة",
        "محام",
        "محاضر",
        "لغوي",
        "سائق شاحنة",
        "خادمة",
        "ساعي بريد",
        "مدير",
        "مدير تسويق",
        "عالم رياضيات",
        "ميكانيكي",
        "تاجر",
        "عالم أرصاد جوية",
        "داية",
        "عارضة أزياء",
        "صراف",
        "موسيقار",
        "مذيع",
        "موزع جرائد",
        "روائي",
        "ممرضة",
        "طبيب عيون",
        "صانع أدوات بصرية",
        "رسام",
        "مدير موظفين",
        "صيدلي",
        "فيلسوف",
        "مصور",
        "عالم فيزياء",
        "معالج طبيعي",
        "طيار",
        "كاتب مسرحي",
        "سمكري",
        "شاعر",
        "ضابط شرطة",
        "شرطي",
        "سياسي",
        "صاحب مطبعة",
        "منتج",
        "مبرمج",
        "طبيب نفساني",
        "ناشر",
        "وكيل عقارات",
        "موظف استقبال",
        "صحفي",
        "مساعد مبيعات",
        "بائع",
        "بائعة",
        "عالم",
        "نحات",
        "سكرتير",
        "سكرتيرة",
        "صباغ أحذية",
        "عامل أحذية",
        "ماسح احذية",
        "عامل بمتجر",
        "صاحب متجر",
        "مطرب",
        "مغني",
        "احصائي",
        "مدير متجر",
        "طبيب جراح",
        "خياط",
        "سائق تاكسي",
        "معلم",
        "موظف بدالة",
        "مترجم",
        "وكيل سفر",
        "سائق شاحنة",
        "خراط",
        "ضارب على الآلة الكاتبة",
        "منجد",
        "نادلة",
        "ساعاتي",
        "عامل",
        "كاتب",
        "خبير في عالم الحيوان",
    )



from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``ar_AA`` locale."""

    word_list = (
        "تاريخ",
        "ثمّة",
        "ويكيبيديا",
        "واشتدّت",
        "عن",
        "أوروبا",
        "التغييرات",
        "كثيرة",
        "سنغافورة",
        "اللازمة",
        "جُل",
        "قد",
        "أن",
        "الشّعبين",
        "للجزر",
        "بالشرقي",
        "ذات",
        "والتي",
        "نهاية",
        "عرفها",
        "وسفن",
        "وبولندا",
        "بالمطالبة",
        "وقوعها",
        "معاملة",
        "العصبة",
        "لمّ",
        "البرية",
        "بعدم",
        "واُسدل",
        "حالية",
        "فمرّ",
        "قبضتهم",
        "وإقامة",
        "الأعمال",
        "معارضة",
        "الإمتعاض",
        "٢٠٠٤",
        "كانت",
        "بينما",
        "شعار",
        "وتزويده",
        "معقل",
        "عشوائية",
        "مدينة",
        "وفي",
        "الأوروبيّون",
        "فكان",
        "ضرب",
        "سكان",
        "الأوروبية",
        "الأمريكي",
        "وبغطاء",
        "الربيع",
        "المضي",
        "جدول",
        "التقليدية",
        "العاصمة",
        "التنازلي",
        "إبّان",
        "بها",
        "الجنوبي",
        "وتنامت",
        "استدعى",
        "الرئيسية",
        "بالتوقيع",
        "بداية",
        "دخول",
        "أثره",
        "الدنمارك",
        "واقتصار",
        "الساحل",
        "الوزراء",
        "الثالث",
        "تونس",
        "الخاسرة",
        "الحيلولة",
        "لفشل",
        "الإطلاق",
        "ماذا",
        "مواقعها",
        "الجديدة",
        "والنفيس",
        "الأرواح",
        "أكثر",
        "الأوربيين",
        "والفرنسي",
        "أوزار",
        "بلديهما",
        "وايرلندا",
        "يتم",
        "جعل",
        "عرض",
        "أفريقيا",
        "إستعمل",
        "التقليدي",
        "عُقر",
        "طوكيو",
        "اتفاقية",
        "بمحاولة",
        "اوروبا",
        "منتصف",
        "أسر",
        "بالرّغم",
        "بضرب",
        "إحكام",
        "أجزاء",
        "الخارجية",
        "كنقطة",
        "بشرية",
        "بقادة",
        "لبلجيكا",
        "استطاعوا",
        "بسبب",
        "تسمّى",
        "الحرة",
        "يرتبط",
        "شاسعة",
        "الواقعة",
        "مارد",
        "والمعدات",
        "لمحاكم",
        "بعلى",
        "التبرعات",
        "ضمنها",
        "فعل",
        "٣٠",
        "الاندونيسية",
        "الى",
        "مكثّفة",
        "وقدّموا",
        "الأثنان",
        "أخرى",
        "العالمي",
        "واحدة",
        "وترك",
        "الحكم",
        "بهناك",
        "بأيدي",
        "باستخدام",
        "الذود",
        "ولكسمبورغ",
        "بعشوائية",
        "الأوضاع",
        "رجوعهم",
        "وأزيز",
        "سياسة",
        "ليبين",
        "رئيس",
        "الجوي",
        "سقطت",
        "تكاليف",
        "وكسبت",
        "وعلى",
        "احداث",
        "شموليةً",
        "إيطاليا",
        "عجّل",
        "أحكم",
        "الدمج",
        "محاولات",
        "يعادل",
        "بريطانيا-فرنسا",
        "ديسمبر",
        "قتيل",
        "التاريخ",
        "استبدال",
        "مقاومة",
        "السيء",
        "الصعداء",
        "أعمال",
        "مسارح",
        "ومضى",
        "تعد",
        "صفحة",
        "النفط",
        "وهولندا",
        "وبريطانيا",
        "جهة",
        "بين",
        "اتّجة",
        "واعتلاء",
        "ومحاولة",
        "المارق",
        "المؤلّفة",
        "شمال",
        "لهذه",
        "بحث",
        "وانهاء",
        "غريمه",
        "وبحلول",
        "وقبل",
        "بتصفح",
        "بال",
        "الفرنسي",
        "وسمّيت",
        "ترتيب",
        "واستمرت",
        "ببلا",
        "الدولارات",
        "إعمار",
        "إعادة",
        "به",
        "الضغوط",
        "حصدت",
        "بقسوة",
        "الطريق",
        "وانتهاءً",
        "حول",
        "اكتوبر",
        "الخاسر",
        "ايطاليا",
        "للأراضي",
        "عالمية",
        "جديدة",
        "أخر",
        "غينيا",
        "قُدُماً",
        "و",
        "مسؤولية",
        "ابتدعها",
        "أمّا",
        "الأول",
        "أدوات",
        "يطول",
        "تُصب",
        "الأولى",
        "المسرح",
        "بهيئة",
        "الخطّة",
        "موالية",
        "مليون",
        "بالجانب",
        "إيو",
        "الهجوم",
        "أعلنت",
        "المدن",
        "علاقة",
        "باستحداث",
        "انتباه",
        "بالولايات",
        "مدن",
        "أساسي",
        "لفرنسا",
        "الجنوب",
        "الثقيل",
        "والعتاد",
        "بقعة",
        "العدّ",
        "اتفاق",
        "تحت",
        "الإحتفاظ",
        "بوقامت",
        "التي",
        "ب٠٨٠٤",
        "ولاتّساع",
        "الا",
        "حكومة",
        "مما",
        "بحيث",
        "مليارات",
        "شواطيء",
        "التحالف",
        "الأوروبي",
        "قام",
        "يتعلّق",
        "مايو",
        "هاربر",
        "العمليات",
        "أما",
        "يعبأ",
        "وشعار",
        "الإقتصادية",
        "والكساد",
        "ليرتفع",
        "وبالرغم",
        "قِبل",
        "الله",
        "تكتيكاً",
        "اعتداء",
        "ب٣٠",
        "تسبب",
        "للحكومة",
        "الأهداف",
        "اللا",
        "ببعض",
        "الستار",
        "وبلجيكا",
        "وفنلندا",
        "ميناء",
        "الساحلية",
        "مقاطعة",
        "وتم",
        "السيطرة",
        "تغييرات",
        "ووصف",
        "الجنود",
        "بلمّ",
        "لدحر",
        "بالحرب",
        "كُلفة",
        "وبدون",
        "وصل",
        "أفاق",
        "الأبرياء",
        "مهمّات",
        "العظمى",
        "والقرى",
        "وأكثرها",
        "لان",
        "مشارف",
        "مساعدة",
        "أضف",
        "مشروط",
        "الكونجرس",
        "للصين",
        "الشمل",
        "اليميني",
        "فكانت",
        "سقوط",
        "بتحت",
        "بكلا",
        "تصرّف",
        "الأثناء",
        "بولندا",
        "واندونيسيا",
        "كلّ",
        "إحتار",
        "الخاطفة",
        "كردة",
        "وحلفاؤها",
        "إتفاقية",
        "وبداية",
        "يذكر",
        "حادثة",
        "مئات",
        "الأسيوي",
        "غير",
        "الانجليزية",
        "استعملت",
        "استمرار",
        "المحيط",
        "وصافرات",
        "وحتّى",
        "الخاصّة",
        "وزارة",
        "بحق",
        "بدارت",
        "ماليزيا",
        "جزيرتي",
        "اليابان",
        "هامش",
        "بوابة",
        "وجهان",
        "وقرى",
        "تجهيز",
        "بولم",
        "البولندي",
        "الجو",
        "لكل",
        "فرنسية",
        "تحرّكت",
        "يقوم",
        "ولم",
        "للإتحاد",
        "الغالي",
        "المشترك",
        "التكاليف",
        "بولاتّساع",
        "الفرنسية",
        "بتحدّي",
        "غرّة",
        "الأرض",
        "مشاركة",
        "فصل",
        "بأم",
        "والكوري",
        "وقد",
        "بـ",
        "كل",
        "بزمام",
        "أي",
        "ماشاء",
        "لغات",
        "اليابانية",
        "ونتج",
        "الضروري",
        "مع",
        "عقبت",
        "لليابان",
        "مسرح",
        "فقامت",
        "بتطويق",
        "بتخصيص",
        "الوراء",
        "تعداد",
        "فبعد",
        "تشكيل",
        "الموسوعة",
        "لكون",
        "واستمر",
        "الأمريكية",
        "إجلاء",
        "بدون",
        "قررت",
        "الشتاء",
        "يتمكن",
        "التخطيط",
        "مكن",
        "بريطانيا",
        "أحدث",
        "تشيكوسلوفاكيا",
        "وبدأت",
        "الشهيرة",
        "حدى",
        "الإيطالية",
        "يتسنّى",
        "اقتصادية",
        "التجارية",
        "بشكل",
        "عسكرياً",
        "على",
        "ألمّ",
        "لبولندا",
        "وسوء",
        "جنوب",
        "بالشتاء",
        "الفترة",
        "للمجهود",
        "أسيا",
        "ارتكبها",
        "بلها",
        "قامت",
        "أوراقهم",
        "بجسيمة",
        "بإعمار",
        "وتتحمّل",
        "القوى",
        "قبل",
        "دنو",
        "عدم",
        "دار",
        "الهادي",
        "خطّة",
        "المجتمع",
        "أملاً",
        "دفّة",
        "بخطوط",
        "السفن",
        "لعدم",
        "حاملات",
        "بل",
        "الإتفاقية",
        "الأمور",
        "بانه",
        "مرجع",
        "النزاع",
        "الدّفاع",
        "بكلّ",
        "ان",
        "السبب",
        "المشتّتون",
        "ودول",
        "ألمانيا",
        "والروسية",
        "والفلبين",
        "بلا",
        "أواخر",
        "ببه",
        "تزامناً",
        "أطراف",
        "يونيو",
        "الأراضي",
        "كانتا",
        "ويعزى",
        "انتهت",
        "بالثالث",
        "وباستثناء",
        "زهاء",
        "الآخر",
        "أخذ",
        "ليركز",
        "يتبقّ",
        "تمهيد",
        "الإقتصادي",
        "بالرغم",
        "سبتمبر",
        "بقد",
        "واتّجه",
        "وصغار",
        "الشتوية",
        "سابق",
        "بعض",
        "الشرق",
        "المتساقطة",
        "الصين",
        "اسبوعين",
        "لهيمنة",
        "الصفحات",
        "أم",
        "الساحة",
        "القادة",
        "لعملة",
        "أمام",
        "تحرير",
        "عليها",
        "وحرمان",
        "حلّت",
        "بأضرار",
        "الثقيلة",
        "وجزر",
        "الأرضية",
        "أراض",
        "والحزب",
        "وباءت",
        "حقول",
        "وبالتحديد",
        "لإنعدام",
        "اعلان",
        "خلاف",
        "انه",
        "بتونس",
        "بمما",
        "الأجل",
        "أوسع",
        "الآلاف",
        "تطوير",
        "اليها",
        "البشريةً",
        "والإتحاد",
        "إختار",
        "وبعض",
        "بالمحور",
        "غضون",
        "شرسة",
        "السادس",
        "الإثنان",
        "لتقليعة",
        "الجنرال",
        "تصفح",
        "ثانية",
        "العناد",
        "لم",
        "لإعادة",
        "سليمان",
        "هناك",
        "جمعت",
        "الباهضة",
        "وبعدما",
        "في",
        "تنفّس",
        "ويتّفق",
        "قدما",
        "المنتصر",
        "جيوب",
        "انتصارهم",
        "قادة",
        "تحرّك",
        "الإمداد",
        "الصينية",
        "جسيمة",
        "تكبّد",
        "وعُرفت",
        "استرجاع",
        "فشكّل",
        "دول",
        "فقد",
        "الطرفين",
        "بالرّد",
        "المعاهدات",
        "الأخذ",
        "لإعلان",
        "كرسي",
        "كلا",
        "هذا",
        "الإنذار",
        "عملية",
        "شيء",
        "ومن",
        "جيما",
        "الشرقية",
        "بالعمل",
        "ما",
        "الولايات",
        "حاول",
        "جديداً",
        "حين",
        "العالم",
        "بنقطة",
        "خيار",
        "بالجوي",
        "والديون",
        "بقيادة",
        "بيكن",
        "يبق",
        "ومطالبة",
        "أراضي",
        "حتى",
        "بأراضي",
        "معزّزة",
        "الثانية",
        "دون",
        "بالسيطرة",
        "نتيجة",
        "دارت",
        "بالإنزال",
        "أمدها",
        "الشهير",
        "من",
        "بلاده",
        "تم",
        "الصفحة",
        "ساعة",
        "الإكتفاء",
        "يكن",
        "عل",
        "أصقاع",
        "بالأجل",
        "تلك",
        "إذ",
        "الإتحاد",
        "المتحدة",
        "إستيلاء",
        "بعرض",
        "بفصل",
        "ممثّلة",
        "شدّت",
        "كان",
        "هنا؟",
        "المبرمة",
        "انذار",
        "تعديل",
        "ليتسنّى",
        "وتنصيب",
        "قائمة",
        "الإنزال",
        "إعلان",
        "للسيطرة",
        "بعد",
        "بالسادس",
        "العالمية",
        "ببحشد",
        "بحشد",
        "إنطلاق",
        "والنرويج",
        "هُزم",
        "فاتّبع",
        "لها",
        "بأسر",
        "الشرقي",
        "المواد",
        "الشمال",
        "الأحمر",
        "٠٨٠٤",
        "ثم",
        "نفس",
        "وقام",
        "عدد",
        "أسابيع",
        "هو",
        "كما",
        "وحتى",
        "المتاخمة",
        "أهّل",
        "أدنى",
        "بفرض",
        "وبعد",
        "المتّبعة",
        "بقصف",
        "الدول",
        "لأداء",
        "جورج",
        "وفرنسا",
        "فرنسا",
        "هذه",
        "الحدود",
        "الشطر",
        "نقطة",
        "بدول",
        "فهرست",
        "حيث",
        "استراليا",
        "وإيطالي",
        "دأبوا",
        "الحكومة",
        "الأمم",
        "فسقط",
        "مرمى",
        "ذلك",
        "مكّن",
        "وإعلان",
        "بمباركة",
        "والمانيا",
    )



from .. import Provider as InternetProvider


class Provider(InternetProvider):

    free_email_domains = (
        "gmail.com",
        "googlemail.com",
        "hotmail.com",
        "suomi24.fi",
        "kolumbus.fi",
        "luukku.com",
        "surffi.net",
    )

    tlds = ("com", "com", "com", "fi", "fi", "net", "org")



# -*- coding: utf-8 -*-

import re

from typing import Optional

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``es_ES`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Spain

    .. |license_plate_unified| replace::
       :meth:`license_plate_unified() <faker.providers.automotive.es_ES.Provider.license_plate_unified>`

    .. |license_plate_by_province| replace::
       :meth:`license_plate_by_province() <faker.providers.automotive.es_ES.Provider.license_plate_by_province>`
    """

    license_formats = (
        # New format
        "#### ???",
    )

    # New format suffix letters (excluding vocals and Q from ascii uppercase)
    license_plate_new_format_suffix_letters = "BCDFGHJKLMNPRSTVWXYZ"

    # Old format suffix letters (excluding Q and R from ascii uppercase)
    license_plate_old_format_suffix_letters = "ABCDEFGHIJKLMNOPSTUVWXYZ"

    # Province prefixes (for old format)
    province_prefix = (
        "A",  # Alicante
        "AB",  # Albacete
        "AL",  # Almería
        "AV",  # Ávila
        "B",  # Barcelona
        "BA",  # Badajoz
        "BI",  # Bilbao
        "BU",  # Burgos
        "C",  # La Coruña
        "CA",  # Cádiz
        "CC",  # Cáceres
        "CS",  # Castellón de la Plana
        "CE",  # Ceuta
        "CO",  # Córdoba
        "CR",  # Ciudad Real
        "CU",  # Cuenca
        "GC",  # Las Palmas (Gran Canaria)
        "GE",  # Girona (until 1992)
        "GI",  # Girona (since 1992)
        "GR",  # Granada
        "GU",  # Guadalajara
        "H",  # Huelva
        "HU",  # Huesca
        "PM",  # Palma de Mallorca (until 1997)
        "IB",  # Islas Baleares (since 1997)
        "J",  # Jaén
        "L",  # Lleida
        "LE",  # León
        "LO",  # Logroño
        "LU",  # Lugo
        "M",  # Madrid
        "MA",  # Málaga
        "ML",  # Melilla
        "MU",  # Murcia
        "O",  # Oviedo
        "OR",  # Ourense (until 1998)
        "OU",  # Ourense (since 1998)
        "P",  # Palencia
        "NA",  # Navarra
        "PO",  # Pontevedra
        "S",  # Santander
        "SA",  # Salamanca
        "SE",  # Sevilla
        "SG",  # Segovia
        "SO",  # Soria
        "SS",  # Donostia/San Sebastián
        "T",  # Tarragona
        "TE",  # Teruel
        "TF",  # Santa Cruz de Tenerife
        "TO",  # Toledo
        "V",  # Valencia
        "VA",  # Valladolid
        "VI",  # Vitoria
        "Z",  # Zaragoza
        "ZA",  # Zamora
    )

    def license_plate_unified(self) -> str:
        """Generate a unified license plate."""
        temp = re.sub(
            r"\?",
            lambda x: self.random_element(self.license_plate_new_format_suffix_letters),
            self.license_formats[0],
        )
        return self.numerify(temp)

    def license_plate_by_province(self, province_prefix: Optional[str] = None) -> str:
        """Generate a provincial license plate.

        If a value for ``province_prefix`` is provided, the value will be used
        as the prefix regardless of validity. If ``None``, then a valid prefix
        will be selected at random.
        """
        province_prefix = province_prefix if province_prefix is not None else self.random_element(self.province_prefix)
        temp = re.sub(
            r"\?",
            lambda x: self.random_element(self.license_plate_old_format_suffix_letters),
            "#### ??",
        )
        return province_prefix + " " + self.numerify(temp)

    def license_plate(self) -> str:
        """Generate a license plate.

        This method randomly chooses (50/50) between |license_plate_unified|
        or |license_plate_by_province| to generate the result.
        """
        if self.generator.random.randint(0, 1):
            return self.license_plate_unified()
        return self.license_plate_by_province()



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):

    DAY_NAMES = {
        "0": "Воскресенье",
        "1": "Понедельник",
        "2": "Вторник",
        "3": "Среда",
        "4": "Четверг",
        "5": "Пятница",
        "6": "Суббота",
    }

    MONTH_NAMES = {
        "01": "Январь",
        "02": "Февраль",
        "03": "Март",
        "04": "Апрель",
        "05": "Май",
        "06": "Июнь",
        "07": "Июль",
        "08": "Август",
        "09": "Сентябрь",
        "10": "Октябрь",
        "11": "Ноябрь",
        "12": "Декабрь",
    }

    # Timezone names are based on Wiki list, source: https://ru.wikipedia.org/wiki/Список_часовых_поясов_по_странам
    countries = [
        {
            "timezones": ["Андорра (UTC+01)"],
            "alpha-2-code": "AD",
            "alpha-3-code": "AND",
            "continent": "Европа",
            "name": "Андорра",
            "capital": "Андорра-ла-Велья",
        },
        {
            "timezones": ["Афганистан (UTC+04:30)"],
            "alpha-2-code": "AF",
            "alpha-3-code": "AFG",
            "continent": "Азия",
            "name": "Афганистан",
            "capital": "Кабул",
        },
        {
            "timezones": ["Антигуа и Барбуда (UTC-04)"],
            "alpha-2-code": "AG",
            "alpha-3-code": "ATG",
            "continent": "Северная Америка",
            "name": "Антигуа и Барбуда",
            "capital": "Сент-Джонс",
        },
        {
            "timezones": ["Албания (UTC+01)"],
            "alpha-2-code": "AL",
            "alpha-3-code": "ALB",
            "continent": "Европа",
            "name": "Албания",
            "capital": "Тирана",
        },
        {
            "timezones": ["Армения (UTC+04)"],
            "alpha-2-code": "AM",
            "alpha-3-code": "ARM",
            "continent": "Азия",
            "name": "Армения",
            "capital": "Ереван",
        },
        {
            "timezones": ["Ангола (UTC+01)"],
            "alpha-2-code": "AO",
            "alpha-3-code": "AGO",
            "continent": "Африка",
            "name": "Ангола",
            "capital": "Луанда",
        },
        {
            "timezones": ["Аргентина (UTC-03)"],
            "alpha-2-code": "AR",
            "alpha-3-code": "ARG",
            "continent": "Южная Америка",
            "name": "Аргентина",
            "capital": "Буэнос Айрес",
        },
        {
            "timezones": ["Австрия (UTC+01)"],
            "alpha-2-code": "AT",
            "alpha-3-code": "AUT",
            "continent": "Европа",
            "name": "Австрия",
            "capital": "Вена",
        },
        {
            "timezones": [
                "Австралия (UTC+05)",
                "Австралия (UTC+06:30)",
                "Австралия (UTC+07)",
                "Австралия (UTC+08)",
                "Австралия (UTC+9:30)",
                "Австралия (UTC+10)",
                "Австралия (UTC+10:30)",
                "Австралия (UTC+11:30)",
            ],
            "alpha-2-code": "AU",
            "alpha-3-code": "AUS",
            "continent": "Океания",
            "name": "Австралия",
            "capital": "Канберра",
        },
        {
            "timezones": ["Азербайджан (UTC+04)"],
            "alpha-2-code": "AZ",
            "alpha-3-code": "AZE",
            "continent": "Азия",
            "name": "Азербайджан",
            "capital": "Баку",
        },
        {
            "timezones": ["Барбадос (UTC-04)"],
            "alpha-2-code": "BB",
            "alpha-3-code": "BRB",
            "continent": "Северная Америка",
            "name": "Барбадос",
            "capital": "Бриджтаун",
        },
        {
            "timezones": ["Бангладеш (UTC+06)"],
            "alpha-2-code": "BD",
            "alpha-3-code": "BGD",
            "continent": "Азия",
            "name": "Бангладеш",
            "capital": "Дака",
        },
        {
            "timezones": ["Бельгия (UTC+01)"],
            "alpha-2-code": "BE",
            "alpha-3-code": "BEL",
            "continent": "Европа",
            "name": "Бельгия",
            "capital": "Брюссель",
        },
        {
            "timezones": ["Буркина-Фасо (UTC)"],
            "alpha-2-code": "BF",
            "alpha-3-code": "BFA",
            "continent": "Африка",
            "name": "Буркина-Фасо",
            "capital": "Уагадугу",
        },
        {
            "timezones": ["Болгария (UTC+02)"],
            "alpha-2-code": "BG",
            "alpha-3-code": "BGR",
            "continent": "Европа",
            "name": "Болгария",
            "capital": "София",
        },
        {
            "timezones": ["Бахрейн (UTC+03)"],
            "alpha-2-code": "BH",
            "alpha-3-code": "BHR",
            "continent": "Азия",
            "name": "Бахрейн",
            "capital": "Манама",
        },
        {
            "timezones": ["Бурунди (UTC+02)"],
            "alpha-2-code": "BI",
            "alpha-3-code": "BDI",
            "continent": "Африка",
            "name": "Бурунди",
            "capital": "Гитега",
        },
        {
            "timezones": ["Бенин (UTC+01)"],
            "alpha-2-code": "BJ",
            "alpha-3-code": "BEN",
            "continent": "Африка",
            "name": "Бенин",
            "capital": "Порто-Ново",
        },
        {
            "timezones": ["Бруней (UTC+08)"],
            "alpha-2-code": "BN",
            "alpha-3-code": "BRN",
            "continent": "Азия",
            "name": "Бруней",
            "capital": "Бандар-Сери-Бегаван",
        },
        {
            "timezones": ["Боливия (UTC-04)"],
            "alpha-2-code": "BO",
            "alpha-3-code": "BOL",
            "continent": "Южная Америка",
            "name": "Боливия",
            "capital": "Сукре",
        },
        {
            "timezones": [
                "Бразилия (UTC-05)",
                "Бразилия (UTC-04)",
                "Бразилия (UTC-03)",
                "Бразилия (UTC-02)",
            ],
            "alpha-2-code": "BR",
            "alpha-3-code": "BRA",
            "continent": "Южная Америка",
            "name": "Бразилия",
            "capital": "Бразилиа",
        },
        {
            "timezones": ["Багамские Острова (UTC-05)"],
            "alpha-2-code": "BS",
            "alpha-3-code": "BHS",
            "continent": "Северная Америка",
            "name": "Багамские Острова",
            "capital": "Нассау",
        },
        {
            "timezones": ["Бутан (UTC+06)"],
            "alpha-2-code": "BT",
            "alpha-3-code": "BTN",
            "continent": "Азия",
            "name": "Бутан",
            "capital": "Тхимпху",
        },
        {
            "timezones": ["Ботсвана (UTC+02)"],
            "alpha-2-code": "BW",
            "alpha-3-code": "BWA",
            "continent": "Африка",
            "name": "Ботсвана",
            "capital": "Габороне",
        },
        {
            "timezones": ["Белоруссия (UTC+03)"],
            "alpha-2-code": "BY",
            "alpha-3-code": "BLR",
            "continent": "Европа",
            "name": "Белоруссия",
            "capital": "Минск",
        },
        {
            "timezones": ["Белиз (UTC-06)"],
            "alpha-2-code": "BZ",
            "alpha-3-code": "BLZ",
            "continent": "Северная Америка",
            "name": "Белиз",
            "capital": "Бельмопан",
        },
        {
            "timezones": [
                "Канада (UTC-08)",
                "Канада (UTC-07)",
                "Канада (UTC-06)",
                "Канада (UTC-05)",
                "Канада (UTC-04)",
                "Канада (UTC-03:30)",
            ],
            "alpha-2-code": "CA",
            "alpha-3-code": "CAN",
            "continent": "Северная Америка",
            "name": "Канада",
            "capital": "Оттава",
        },
        {
            "timezones": [
                "Демократическая Республика Конго (UTC+01)",
                "Демократическая Республика Конго (UTC+02)",
            ],
            "alpha-2-code": "CD",
            "alpha-3-code": "COD",
            "continent": "Африка",
            "name": "Демократическая Республика Конго",
            "capital": "Киншаса",
        },
        {
            "timezones": ["Республика Конго (UTC+01)"],
            "alpha-2-code": "CG",
            "alpha-3-code": "COG",
            "continent": "Африка",
            "name": "Руспублика Конго",
            "capital": "Браззавиль",
        },
        {
            "timezones": ["Кот-д'Ивуар (UTC)"],
            "alpha-2-code": "CI",
            "alpha-3-code": "CIV",
            "continent": "Африка",
            "name": "Кот-д'Ивуар",
            "capital": "Ямусукро",
        },
        {
            "timezones": ["Чили (UTC-06)", "Чили (UTC-04)"],
            "alpha-2-code": "CL",
            "alpha-3-code": "CHL",
            "continent": "Южная Америка",
            "name": "Чили",
            "capital": "Сантьяго",
        },
        {
            "timezones": ["Камерун (UTC+01)"],
            "alpha-2-code": "CM",
            "alpha-3-code": "CMR",
            "continent": "Африка",
            "name": "Камерун",
            "capital": "Яунде",
        },
        {
            "timezones": ["Китай (UTC+08)"],
            "alpha-2-code": "CN",
            "alpha-3-code": "CHN",
            "continent": "Азия",
            "name": "Китайская Народная Республика",
            "capital": "Пекин",
        },
        {
            "timezones": ["Колумбия (UTC-05)"],
            "alpha-2-code": "CO",
            "alpha-3-code": "COL",
            "continent": "Южная Америка",
            "name": "Колумбия",
            "capital": "Богота",
        },
        {
            "timezones": ["Коста-Рика (UTC-06)"],
            "alpha-2-code": "CR",
            "alpha-3-code": "CRI",
            "continent": "Северная Америка",
            "name": "Коста-Рика",
            "capital": "Сан-Хосе",
        },
        {
            "timezones": ["Куба (UTC-05)"],
            "alpha-2-code": "CU",
            "alpha-3-code": "CUB",
            "continent": "Северная Америка",
            "name": "Куба",
            "capital": "Гавана",
        },
        {
            "timezones": ["Кабо-Верде (UTC-01)"],
            "alpha-2-code": "CV",
            "alpha-3-code": "CPV",
            "continent": "Африка",
            "name": "Кабо-Верде",
            "capital": "Прая",
        },
        {
            "timezones": ["Кипр (UTC+02)"],
            "alpha-2-code": "CY",
            "alpha-3-code": "CYP",
            "continent": "Азия",
            "name": "Кипр",
            "capital": "Никосия",
        },
        {
            "timezones": ["Чехия (UTC+01)"],
            "alpha-2-code": "CZ",
            "alpha-3-code": "CZE",
            "continent": "Европа",
            "name": "Чехия",
            "capital": "Прага",
        },
        {
            "timezones": ["Германия (UTC+01)"],
            "alpha-2-code": "DE",
            "alpha-3-code": "DEU",
            "continent": "Европа",
            "name": "Германия",
            "capital": "Берлин",
        },
        {
            "timezones": ["Джибути (UTC+03)"],
            "alpha-2-code": "DJ",
            "alpha-3-code": "DJI",
            "continent": "Африка",
            "name": "Джибути",
            "capital": "Джибути",
        },
        {
            "timezones": ["Дания (UTC+01)"],
            "alpha-2-code": "DK",
            "alpha-3-code": "DNK",
            "continent": "Европа",
            "name": "Дания",
            "capital": "Копенгаген",
        },
        {
            "timezones": ["Доминика (UTC-04)"],
            "alpha-2-code": "DM",
            "alpha-3-code": "DMA",
            "continent": "Северная Америка",
            "name": "Доминика",
            "capital": "Розо",
        },
        {
            "timezones": ["Доминиканская Республика (UTC-04)"],
            "alpha-2-code": "DO",
            "alpha-3-code": "DOM",
            "continent": "Северная Америка",
            "name": "Доминиканская Республика",
            "capital": "Санто-Доминго",
        },
        {
            "timezones": ["Эквадор (UTC-06)", "Эквадор (UTC-05)"],
            "alpha-2-code": "EC",
            "alpha-3-code": "ECU",
            "continent": "Южная Америка",
            "name": "Эквадор",
            "capital": "Кито",
        },
        {
            "timezones": ["Эстония (UTC+02)"],
            "alpha-2-code": "EE",
            "alpha-3-code": "EST",
            "continent": "Европа",
            "name": "Эстония",
            "capital": "Таллинн",
        },
        {
            "timezones": ["Египет (UTC+02)"],
            "alpha-2-code": "EG",
            "alpha-3-code": "EGY",
            "continent": "Африка",
            "name": "Египет",
            "capital": "Каир",
        },
        {
            "timezones": ["Эритрея (UTC+03)"],
            "alpha-2-code": "ER",
            "alpha-3-code": "ERI",
            "continent": "Африка",
            "name": "Эритрея",
            "capital": "Асмэра",
        },
        {
            "timezones": ["Эфиопия (UTC+03)"],
            "alpha-2-code": "ET",
            "alpha-3-code": "ETH",
            "continent": "Африка",
            "name": "Эфиопия",
            "capital": "Аддис-Абеба",
        },
        {
            "timezones": ["Финляндия (UTC+02)"],
            "alpha-2-code": "FI",
            "alpha-3-code": "FIN",
            "continent": "Европа",
            "name": "Финляндия",
            "capital": "Хельсинки",
        },
        {
            "timezones": ["Фиджи (UTC+12)"],
            "alpha-2-code": "FJ",
            "alpha-3-code": "FJI",
            "continent": "Океания",
            "name": "Фиджи",
            "capital": "Сува",
        },
        {
            "timezones": ["Франция (UTC+01)"],
            "alpha-2-code": "FR",
            "alpha-3-code": "FRA",
            "continent": "Европа",
            "name": "Франция",
            "capital": "Париж",
        },
        {
            "timezones": ["Габон (UTC+01)"],
            "alpha-2-code": "GA",
            "alpha-3-code": "GAB",
            "continent": "Африка",
            "name": "Габон",
            "capital": "Либревиль",
        },
        {
            "timezones": ["Грузия (UTC+04)"],
            "alpha-2-code": "GE",
            "alpha-3-code": "GEO",
            "continent": "Азия",
            "name": "Грузия",
            "capital": "Тбилиси",
        },
        {
            "timezones": ["Гана (UTC)"],
            "alpha-2-code": "GH",
            "alpha-3-code": "GHA",
            "continent": "Африка",
            "name": "Гана",
            "capital": "Аккра",
        },
        {
            "timezones": ["Гамбия (UTC)"],
            "alpha-2-code": "GM",
            "alpha-3-code": "GMB",
            "continent": "Африка",
            "name": "Гамбия",
            "capital": "Банджул",
        },
        {
            "timezones": ["Гвинея (UTC)"],
            "alpha-2-code": "GN",
            "alpha-3-code": "GIN",
            "continent": "Африка",
            "name": "Гвинея",
            "capital": "Конакри",
        },
        {
            "timezones": ["Греция (UTC+02)"],
            "alpha-2-code": "GR",
            "alpha-3-code": "GRC",
            "continent": "Европа",
            "name": "Греция",
            "capital": "Афины",
        },
        {
            "timezones": ["Гватемала (UTC-06)"],
            "alpha-2-code": "GT",
            "alpha-3-code": "GTM",
            "continent": "Северная Америка",
            "name": "Гватемала",
            "capital": "Гватемала",
        },
        {
            "timezones": ["Гаити (UTC-05)"],
            "alpha-2-code": "HT",
            "alpha-3-code": "HTI",
            "continent": "Северная Америка",
            "name": "Гаити",
            "capital": "Порт-о-Пренс",
        },
        {
            "timezones": ["Гвинея-Бисау (UTC)"],
            "alpha-2-code": "GW",
            "alpha-3-code": "GNB",
            "continent": "Африка",
            "name": "Гвинея-Бисау",
            "capital": "Бисау",
        },
        {
            "timezones": ["Гайана (UTC-04)"],
            "alpha-2-code": "GY",
            "alpha-3-code": "GUY",
            "continent": "Южная Америка",
            "name": "Гайана",
            "capital": "Джорджтаун",
        },
        {
            "timezones": ["Гондурас (UTC-06)"],
            "alpha-2-code": "HN",
            "alpha-3-code": "HND",
            "continent": "Северная Америка",
            "name": "Гондурас",
            "capital": "Тегусигальпа",
        },
        {
            "timezones": ["Венгрия (UTC+01)"],
            "alpha-2-code": "HU",
            "alpha-3-code": "HUN",
            "continent": "Европа",
            "name": "Венгрия",
            "capital": "Будапешт",
        },
        {
            "timezones": [
                "Индонезия (UTC+07)",
                "Индонезия (UTC+08)",
                "Индонезия (UTC+09)",
            ],
            "alpha-2-code": "ID",
            "alpha-3-code": "IDN",
            "continent": "Азия",
            "name": "Индонезия",
            "capital": "Джакарта",
        },
        {
            "timezones": ["Ирландия (UTC)"],
            "alpha-2-code": "IE",
            "alpha-3-code": "IRL",
            "continent": "Европа",
            "name": "Ирландия",
            "capital": "Дублин",
        },
        {
            "timezones": ["Израиль (UTC+02)"],
            "alpha-2-code": "IL",
            "alpha-3-code": "ISR",
            "continent": "Азия",
            "name": "Израиль",
            "capital": "Иерусалим",
        },
        {
            "timezones": ["Индия (UTC+05:30"],
            "alpha-2-code": "IN",
            "alpha-3-code": "IND",
            "continent": "Азия",
            "name": "Индия",
            "capital": "Дели",
        },
        {
            "timezones": ["Ирак (UTC+03)"],
            "alpha-2-code": "IQ",
            "alpha-3-code": "IRQ",
            "continent": "Азия",
            "name": "Ирак",
            "capital": "Багдад",
        },
        {
            "timezones": ["Иран (UTC+03:30)"],
            "alpha-2-code": "IR",
            "alpha-3-code": "IRN",
            "continent": "Азия",
            "name": "Иран",
            "capital": "Тегеран",
        },
        {
            "timezones": ["Исландия (UTC)"],
            "alpha-2-code": "IS",
            "alpha-3-code": "ISL",
            "continent": "Европа",
            "name": "Исландия",
            "capital": "Рейкьявик",
        },
        {
            "timezones": ["Италия (UTC+01)"],
            "alpha-2-code": "IT",
            "alpha-3-code": "ITA",
            "continent": "Европа",
            "name": "Италия",
            "capital": "Рим",
        },
        {
            "timezones": ["Ямайка (UTC-05)"],
            "alpha-2-code": "JM",
            "alpha-3-code": "JAM",
            "continent": "Северная Америка",
            "name": "Ямайка",
            "capital": "Кингстон",
        },
        {
            "timezones": ["Иордания (UTC+02)"],
            "alpha-2-code": "JO",
            "alpha-3-code": "JOR",
            "continent": "Азия",
            "name": "Иордания",
            "capital": "Амман",
        },
        {
            "timezones": ["Япония (UTC+09)"],
            "alpha-2-code": "JP",
            "alpha-3-code": "JPN",
            "continent": "Азия",
            "name": "Япония",
            "capital": "Токио",
        },
        {
            "timezones": ["Кения (UTC+03)"],
            "alpha-2-code": "KE",
            "alpha-3-code": "KEN",
            "continent": "Африка",
            "name": "Кения",
            "capital": "Найроби",
        },
        {
            "timezones": ["Киргизия (UTC+06)"],
            "alpha-2-code": "KG",
            "alpha-3-code": "KGZ",
            "continent": "Азия",
            "name": "Киргизия",
            "capital": "Бишкек",
        },
        {
            "timezones": [
                "Кирибати (UTC+12)",
                "Кирибати (UTC+13)",
                "Кирибати (UTC+14)",
            ],
            "alpha-2-code": "KI",
            "alpha-3-code": "KIR",
            "continent": "Океания",
            "name": "Кирибати",
            "capital": "Южная Тарава",
        },
        {
            "timezones": ["КНДР (UTC+09)"],
            "alpha-2-code": "KP",
            "alpha-3-code": "PRK",
            "continent": "Азия",
            "name": "КНДР",
            "capital": "Пхеньян",
        },
        {
            "timezones": ["Республика Корея (UTC+09)"],
            "alpha-2-code": "KR",
            "alpha-3-code": "KOR",
            "continent": "Азия",
            "name": "Республика Корея",
            "capital": "Сеул",
        },
        {
            "timezones": ["Кувейт (UTC+03)"],
            "alpha-2-code": "KW",
            "alpha-3-code": "KWT",
            "continent": "Азия",
            "name": "Кувейт",
            "capital": "Эль-Кувейт",
        },
        {
            "timezones": ["Ливан (UTC+02)"],
            "alpha-2-code": "LB",
            "alpha-3-code": "LBN",
            "continent": "Азия",
            "name": "Ливан",
            "capital": "Бейрут",
        },
        {
            "timezones": ["Лихтенштейн (UTC+01)"],
            "alpha-2-code": "LI",
            "alpha-3-code": "LIE",
            "continent": "Европа",
            "name": "Лихтенштейн",
            "capital": "Вадуц",
        },
        {
            "timezones": ["Либерия (UTC)"],
            "alpha-2-code": "LR",
            "alpha-3-code": "LBR",
            "continent": "Африка",
            "name": "Либерия",
            "capital": "Монровия",
        },
        {
            "timezones": ["Лесото (UTC+02)"],
            "alpha-2-code": "LS",
            "alpha-3-code": "LSO",
            "continent": "Африка",
            "name": "Лесото",
            "capital": "Масеру",
        },
        {
            "timezones": ["Литва (UTC+02)"],
            "alpha-2-code": "LT",
            "alpha-3-code": "LTU",
            "continent": "Европа",
            "name": "Литва",
            "capital": "Вильнюс",
        },
        {
            "timezones": ["Люксембург (UTC+01)"],
            "alpha-2-code": "LU",
            "alpha-3-code": "LUX",
            "continent": "Европа",
            "name": "Люксембург",
            "capital": "Люксембург",
        },
        {
            "timezones": ["Латвия (UTC+02)"],
            "alpha-2-code": "LV",
            "alpha-3-code": "LVA",
            "continent": "Европа",
            "name": "Латвия",
            "capital": "Рига",
        },
        {
            "timezones": ["Ливия (UTC+02)"],
            "alpha-2-code": "LY",
            "alpha-3-code": "LBY",
            "continent": "Африка",
            "name": "Ливия",
            "capital": "Триполи",
        },
        {
            "timezones": ["Мадагаскар (UTC+03)"],
            "alpha-2-code": "MG",
            "alpha-3-code": "MDG",
            "continent": "Африка",
            "name": "Мадагаскар",
            "capital": "Антананариву",
        },
        {
            "timezones": ["Маршалловы Острова (UTC+12)"],
            "alpha-2-code": "MH",
            "alpha-3-code": "MHL",
            "continent": "Океания",
            "name": "Маршалловы Острова",
            "capital": "Маджуро",
        },
        {
            "timezones": ["Северная Македония (UTC+01)"],
            "alpha-2-code": "MK",
            "alpha-3-code": "MKD",
            "continent": "Европа",
            "name": "Северная Македония",
            "capital": "Скопье",
        },
        {
            "timezones": ["Мали (UTC)"],
            "alpha-2-code": "ML",
            "alpha-3-code": "MLI",
            "continent": "Африка",
            "name": "Мали",
            "capital": "Бамако",
        },
        {
            "timezones": ["Мьянма (UTC+06:30)"],
            "alpha-2-code": "MM",
            "alpha-3-code": "MMR",
            "continent": "Азия",
            "name": "Мьянма",
            "capital": "Нейпьидо",
        },
        {
            "timezones": ["Монголия (UTC+07)", "Монголия (UTC+08)"],
            "alpha-2-code": "MN",
            "alpha-3-code": "MNG",
            "continent": "Азия",
            "name": "Монголия",
            "capital": "Улан-Батор",
        },
        {
            "timezones": ["Мавритания (UTC)"],
            "alpha-2-code": "MR",
            "alpha-3-code": "MRT",
            "continent": "Африка",
            "name": "Мавритания",
            "capital": "Нуакшот",
        },
        {
            "timezones": ["Мальта (UTC+01)"],
            "alpha-2-code": "MT",
            "alpha-3-code": "MLT",
            "continent": "Европа",
            "name": "Мальта",
            "capital": "Валлетта",
        },
        {
            "timezones": ["Маврикий (UTC+04)"],
            "alpha-2-code": "MU",
            "alpha-3-code": "MUS",
            "continent": "Африка",
            "name": "Маврикий",
            "capital": "Порт-Луи",
        },
        {
            "timezones": ["Мальдивы (UTC+05)"],
            "alpha-2-code": "MV",
            "alpha-3-code": "MDV",
            "continent": "Азия",
            "name": "Мальдивы",
            "capital": "Мале",
        },
        {
            "timezones": ["Малави (UTC+02)"],
            "alpha-2-code": "MW",
            "alpha-3-code": "MWI",
            "continent": "Африка",
            "name": "Малави",
            "capital": "Лилонгве",
        },
        {
            "timezones": ["Мексика (UTC-08)", "Мексика (UTC-07)", "Мексика (UTC-06)"],
            "alpha-2-code": "MX",
            "alpha-3-code": "MEX",
            "continent": "Северная Америка",
            "name": "Мексика",
            "capital": "Мехико",
        },
        {
            "timezones": ["Малайзия (UTC+08)"],
            "alpha-2-code": "MY",
            "alpha-3-code": "MYS",
            "continent": "Азия",
            "name": "Малайзия",
            "capital": "Куала-Лумпур",
        },
        {
            "timezones": ["Мозамбик (UTC+02)"],
            "alpha-2-code": "MZ",
            "alpha-3-code": "MOZ",
            "continent": "Африка",
            "name": "Мозамбик",
            "capital": "Мапуту",
        },
        {
            "timezones": ["Намибия (UTC+01)"],
            "alpha-2-code": "NA",
            "alpha-3-code": "NAM",
            "continent": "Африка",
            "name": "Намибия",
            "capital": "Виндхук",
        },
        {
            "timezones": ["Нигер (UTC+01)"],
            "alpha-2-code": "NE",
            "alpha-3-code": "NER",
            "continent": "Африка",
            "name": "Нигер",
            "capital": "Ниамей",
        },
        {
            "timezones": ["Нигерия (UTC+01)"],
            "alpha-2-code": "NG",
            "alpha-3-code": "NGA",
            "continent": "Африка",
            "name": "Нигерия",
            "capital": "Абуджа",
        },
        {
            "timezones": ["Никарагуа (UTC-06)"],
            "alpha-2-code": "NI",
            "alpha-3-code": "NIC",
            "continent": "Северная Америка",
            "name": "Никарагуа",
            "capital": "Манагуа",
        },
        {
            "timezones": ["Нидерланды (UTC+01)"],
            "alpha-2-code": "NL",
            "alpha-3-code": "NLD",
            "continent": "Европа",
            "name": "Нидерланды",
            "capital": "Амстердам",
        },
        {
            "timezones": ["Норвегия (UTC+01)"],
            "alpha-2-code": "NO",
            "alpha-3-code": "NOR",
            "continent": "Европа",
            "name": "Норвегия",
            "capital": "Осло",
        },
        {
            "timezones": ["Непал (UTC+05:45"],
            "alpha-2-code": "NP",
            "alpha-3-code": "NPL",
            "continent": "Азия",
            "name": "Непал",
            "capital": "Катманду",
        },
        {
            "timezones": ["Науру (UTC+12)"],
            "alpha-2-code": "NR",
            "alpha-3-code": "NRU",
            "continent": "Океания",
            "name": "Науру",
            "capital": "Ярен",
        },
        {
            "timezones": ["Новая Зеландия (UTC+12)"],
            "alpha-2-code": "NZ",
            "alpha-3-code": "NZL",
            "continent": "Океания",
            "name": "Новая Зеландия",
            "capital": "Веллингтон",
        },
        {
            "timezones": ["Оман (UTC+04"],
            "alpha-2-code": "OM",
            "alpha-3-code": "OMN",
            "continent": "Азия",
            "name": "Оман",
            "capital": "Маскат",
        },
        {
            "timezones": ["Панама (UTC-05)"],
            "alpha-2-code": "PA",
            "alpha-3-code": "PAN",
            "continent": "Северная Америка",
            "name": "Панама",
            "capital": "Панама",
        },
        {
            "timezones": ["Перу (UTC-05)"],
            "alpha-2-code": "PE",
            "alpha-3-code": "PER",
            "continent": "Южная Америка",
            "name": "Перу",
            "capital": "Лима",
        },
        {
            "timezones": ["Папуа - Новая Гвинея (UTC+10)"],
            "alpha-2-code": "PG",
            "alpha-3-code": "PNG",
            "continent": "Океания",
            "name": "Папуа - Новая Гвинея",
            "capital": "Порт-Морсби",
        },
        {
            "timezones": ["Филиппины (UTC+08)"],
            "alpha-2-code": "PH",
            "alpha-3-code": "PHL",
            "continent": "Азия",
            "name": "Филиппины",
            "capital": "Манила",
        },
        {
            "timezones": ["Пакистан (UTC+05)"],
            "alpha-2-code": "PK",
            "alpha-3-code": "PAK",
            "continent": "Азия",
            "name": "Пакистан",
            "capital": "Исламабад",
        },
        {
            "timezones": ["Польша (UTC+01)"],
            "alpha-2-code": "PL",
            "alpha-3-code": "POL",
            "continent": "Европа",
            "name": "Польша",
            "capital": "Варшава",
        },
        {
            "timezones": ["Португалия (UTC)"],
            "alpha-2-code": "PT",
            "alpha-3-code": "PRT",
            "continent": "Европа",
            "name": "Португалия",
            "capital": "Лиссабон",
        },
        {
            "timezones": ["Палау (UTC+09)"],
            "alpha-2-code": "PW",
            "alpha-3-code": "PLW",
            "continent": "Океания",
            "name": "Палау",
            "capital": "Кампала",
        },
        {
            "timezones": ["Парагвай (UTC-04)"],
            "alpha-2-code": "PY",
            "alpha-3-code": "PRY",
            "continent": "Южная Америка",
            "name": "Парагвай",
            "capital": "Асунсьон",
        },
        {
            "timezones": ["Катар (UTC+03)"],
            "alpha-2-code": "QA",
            "alpha-3-code": "QAT",
            "continent": "Азия",
            "name": "Катар",
            "capital": "Доха",
        },
        {
            "timezones": ["Румыния (UTC+02)"],
            "alpha-2-code": "RO",
            "alpha-3-code": "ROU",
            "continent": "Европа",
            "name": "Румыния",
            "capital": "Бухарест",
        },
        {
            "timezones": [
                "Россия (UTC+02)",
                "Россия (UTC+03)",
                "Россия (UTC+04)",
                "Россия (UTC+05)",
                "Россия (UTC+06)",
                "Россия (UTC+07)",
                "Россия (UTC+08)",
                "Россия (UTC+09)",
                "Россия (UTC+10)",
                "Россия (UTC+11)",
                "Россия (UTC+12)",
            ],
            "alpha-2-code": "RU",
            "alpha-3-code": "RUS",
            "continent": "Европа",
            "name": "Россия",
            "capital": "Москва",
        },
        {
            "timezones": ["Руанда (UTC+02)"],
            "alpha-2-code": "RW",
            "alpha-3-code": "RWA",
            "continent": "Африка",
            "name": "Руанда",
            "capital": "Кигали",
        },
        {
            "timezones": ["Саудовская Аравия (UTC+03)"],
            "alpha-2-code": "SA",
            "alpha-3-code": "SAU",
            "continent": "Азия",
            "name": "Саудовская Аравия",
            "capital": "Эр-Рияд",
        },
        {
            "timezones": ["Соломоновы Острова (UTC+11)"],
            "alpha-2-code": "SB",
            "alpha-3-code": "SLB",
            "continent": "Океания",
            "name": "Соломоновы Острова",
            "capital": "Хониара",
        },
        {
            "timezones": ["Сейшельские острова (UTC+04)"],
            "alpha-2-code": "SC",
            "alpha-3-code": "SYC",
            "continent": "Африка",
            "name": "Сейшельские острова",
            "capital": "Виктория",
        },
        {
            "timezones": ["Судан (UTC+03)"],
            "alpha-2-code": "SD",
            "alpha-3-code": "SDN",
            "continent": "Африка",
            "name": "Судан",
            "capital": "Хартум",
        },
        {
            "timezones": ["Швеция (UTC+01)"],
            "alpha-2-code": "SE",
            "alpha-3-code": "SWE",
            "continent": "Европа",
            "name": "Швеци",
            "capital": "Стокгольм",
        },
        {
            "timezones": ["Сингапур (UTC+08)"],
            "alpha-2-code": "SG",
            "alpha-3-code": "SGP",
            "continent": "Азия",
            "name": "Сингапур",
            "capital": "Сингапур",
        },
        {
            "timezones": ["Словения (UTC+01)"],
            "alpha-2-code": "SI",
            "alpha-3-code": "SVN",
            "continent": "Европа",
            "name": "Словения",
            "capital": "Любляна",
        },
        {
            "timezones": ["Словакия (UTC+01)"],
            "alpha-2-code": "SK",
            "alpha-3-code": "SVK",
            "continent": "Европа",
            "name": "Словакия",
            "capital": "Братислава",
        },
        {
            "timezones": ["Сьерра-Леоне (UTC)"],
            "alpha-2-code": "SL",
            "alpha-3-code": "SLE",
            "continent": "Африка",
            "name": "Сьерра Леоне",
            "capital": "Фритаун",
        },
        {
            "timezones": ["Сан-Марино (UTC+01)"],
            "alpha-2-code": "SM",
            "alpha-3-code": "SMR",
            "continent": "Европа",
            "name": "Сан-Марино",
            "capital": "Сан-Марино",
        },
        {
            "timezones": ["Сенегал (UTC)"],
            "alpha-2-code": "SN",
            "alpha-3-code": "SEN",
            "continent": "Африка",
            "name": "Сенегал",
            "capital": "Дакар",
        },
        {
            "timezones": ["Сомали (UTC+03)"],
            "alpha-2-code": "SO",
            "alpha-3-code": "SOM",
            "continent": "Африка",
            "name": "Сомали",
            "capital": "Могадишо",
        },
        {
            "timezones": ["Суринам (UTC-03)"],
            "alpha-2-code": "SR",
            "alpha-3-code": "SUR",
            "continent": "Южная Америка",
            "name": "Суринам",
            "capital": "Парамарибо",
        },
        {
            "timezones": ["Сан-Томе и Принсипи (UTC)"],
            "alpha-2-code": "ST",
            "alpha-3-code": "STP",
            "continent": "Африка",
            "name": "Сан-Томе и Принсипи",
            "capital": "Сан-Томе",
        },
        {
            "timezones": ["Сирия (UTC+02)"],
            "alpha-2-code": "SY",
            "alpha-3-code": "SYR",
            "continent": "Азия",
            "name": "Сирия",
            "capital": "Дамаск",
        },
        {
            "timezones": ["Того (UTC)"],
            "alpha-2-code": "TG",
            "alpha-3-code": "TGO",
            "continent": "Африка",
            "name": "Того",
            "capital": "Ломе",
        },
        {
            "timezones": ["Таиланд (UTC+07)"],
            "alpha-2-code": "TH",
            "alpha-3-code": "THA",
            "continent": "Азия",
            "name": "Таиланд",
            "capital": "Бангкок",
        },
        {
            "timezones": ["Таджикистан (UTC+05)"],
            "alpha-2-code": "TJ",
            "alpha-3-code": "TJK",
            "continent": "Азия",
            "name": "Таджикистан",
            "capital": "Душанбе",
        },
        {
            "timezones": ["Туркмения (UTC+05)"],
            "alpha-2-code": "TM",
            "alpha-3-code": "TKM",
            "continent": "Азия",
            "name": "Туркмения",
            "capital": "Ашхабад",
        },
        {
            "timezones": ["Тунис (UTC+01)"],
            "alpha-2-code": "TN",
            "alpha-3-code": "TUN",
            "continent": "Африка",
            "name": "Тунис",
            "capital": "Тунис",
        },
        {
            "timezones": ["Тонга (UTC+13)"],
            "alpha-2-code": "TO",
            "alpha-3-code": "TON",
            "continent": "Океания",
            "name": "Тонга",
            "capital": "Нукуалофа",
        },
        {
            "timezones": ["Турция (UTC+02)"],
            "alpha-2-code": "TR",
            "alpha-3-code": "TUR",
            "continent": "Азия",
            "name": "Турция",
            "capital": "Анкара",
        },
        {
            "timezones": ["Тринидад и Тобаго (UTC-04)"],
            "alpha-2-code": "TT",
            "alpha-3-code": "TTO",
            "continent": "Северная Америка",
            "name": "Тринидад и Тобаго",
            "capital": "Порт-оф-Спейн",
        },
        {
            "timezones": ["Тувалу (UTC+12)"],
            "alpha-2-code": "TV",
            "alpha-3-code": "TUV",
            "continent": "Океания",
            "name": "Тувалу",
            "capital": "Фунафути",
        },
        {
            "timezones": ["Танзания (UTC+03)"],
            "alpha-2-code": "TZ",
            "alpha-3-code": "TZA",
            "continent": "Африка",
            "name": "Танзания",
            "capital": "Додома",
        },
        {
            "timezones": ["Украина (UTC+02)", "Украина (UTC+03)"],
            "alpha-2-code": "UA",
            "alpha-3-code": "UKR",
            "continent": "Европа",
            "name": "Украина",
            "capital": "Киев",
        },
        {
            "timezones": ["Уганда (UTC+03)"],
            "alpha-2-code": "UG",
            "alpha-3-code": "UGA",
            "continent": "Африка",
            "name": "Уганда",
            "capital": "Кампала",
        },
        {
            "timezones": [
                "США (UTC-11)",
                "США (UTC-10)",
                "США (UTC-09)",
                "США (UTC-08)",
                "США (UTC-07)",
                "США (UTC-06)",
                "США (UTC-05)",
                "США (UTC-04)",
                "США (UTC+10)",
            ],
            "alpha-2-code": "US",
            "alpha-3-code": "USA",
            "continent": "Северная Америка",
            "name": "США",
            "capital": "Вашингтон",
        },
        {
            "timezones": ["Уругвай (UTC-03)"],
            "alpha-2-code": "UY",
            "alpha-3-code": "URY",
            "continent": "Южная Америка",
            "name": "Уругвай",
            "capital": "Монтевидео",
        },
        {
            "timezones": ["Узбекистан (UTC+05)"],
            "alpha-2-code": "UZ",
            "alpha-3-code": "UZB",
            "continent": "Азия",
            "name": "Узбекистан",
            "capital": "Ташкент",
        },
        {
            "timezones": ["Ватикан (UTC+01)"],
            "alpha-2-code": "VA",
            "alpha-3-code": "VAT",
            "continent": "Европа",
            "name": "Ватикан",
            "capital": "Ватикан",
        },
        {
            "timezones": ["Венесуэла (UTC-04:30)"],
            "alpha-2-code": "VE",
            "alpha-3-code": "VEN",
            "continent": "Южная Америка",
            "name": "Венесуэла",
            "capital": "Каракас",
        },
        {
            "timezones": ["Вьетнам (UTC+07)"],
            "alpha-2-code": "VN",
            "alpha-3-code": "VNM",
            "continent": "Азия",
            "name": "Вьетнам",
            "capital": "Ханой",
        },
        {
            "timezones": ["Вануату (UTC+11)"],
            "alpha-2-code": "VU",
            "alpha-3-code": "VUT",
            "continent": "Океания",
            "name": "Вануату",
            "capital": "Порт-Вила",
        },
        {
            "timezones": ["Йемен (UTC+03)"],
            "alpha-2-code": "YE",
            "alpha-3-code": "YEM",
            "continent": "Азия",
            "name": "Йемен",
            "capital": "Сана",
        },
        {
            "timezones": ["Замбия (UTC+02)"],
            "alpha-2-code": "ZM",
            "alpha-3-code": "ZMB",
            "continent": "Африка",
            "name": "Замбия",
            "capital": "Лусака",
        },
        {
            "timezones": ["Зимбабве (UTC+02)"],
            "alpha-2-code": "ZW",
            "alpha-3-code": "ZWE",
            "continent": "Африка",
            "name": "Зимбабве",
            "capital": "Хараре",
        },
        {
            "timezones": ["Алжир (UTC+01)"],
            "alpha-2-code": "DZ",
            "alpha-3-code": "DZA",
            "continent": "Африка",
            "name": "Алжир",
            "capital": "Алжир",
        },
        {
            "timezones": ["Босния и Герцеговина (UTC+01)"],
            "alpha-2-code": "BA",
            "alpha-3-code": "BIH",
            "continent": "Европа",
            "name": "Босния и Герцеговина",
            "capital": "Сараево",
        },
        {
            "timezones": ["Камбоджа (UTC+07)"],
            "alpha-2-code": "KH",
            "alpha-3-code": "KHM",
            "continent": "Азия",
            "name": "Камбоджа",
            "capital": "Пномпень",
        },
        {
            "timezones": ["ЦАР (UTC+01)"],
            "alpha-2-code": "CF",
            "alpha-3-code": "CAF",
            "continent": "Африка",
            "name": "ЦАР",
            "capital": "Банги",
        },
        {
            "timezones": ["Чад (UTC+01)"],
            "alpha-2-code": "TD",
            "alpha-3-code": "TCD",
            "continent": "Африка",
            "name": "Чад",
            "capital": "Нджамена",
        },
        {
            "timezones": ["Коморы (UTC+03)"],
            "alpha-2-code": "KM",
            "alpha-3-code": "COM",
            "continent": "Африка",
            "name": "Коморы",
            "capital": "Морони",
        },
        {
            "timezones": ["Хорватия (UTC+01)"],
            "alpha-2-code": "HR",
            "alpha-3-code": "HRV",
            "continent": "Европа",
            "name": "Хорватия",
            "capital": "Загреб",
        },
        {
            "timezones": ["Восточный Тимор (UTC+09)"],
            "alpha-2-code": "TL",
            "alpha-3-code": "TLS",
            "continent": "Азия",
            "name": "Восточный Тимор",
            "capital": "Дили",
        },
        {
            "timezones": ["Сальвадор (UTC-06)"],
            "alpha-2-code": "SV",
            "alpha-3-code": "SLV",
            "continent": "Северная Америка",
            "name": "Сальвадор",
            "capital": "Сан-Сальвадор",
        },
        {
            "timezones": ["Экваториальная Гвинея (UTC+01)"],
            "alpha-2-code": "GQ",
            "alpha-3-code": "GNQ",
            "continent": "Африка",
            "name": "Экваториальная Гвинея",
            "capital": "Малабо",
        },
        {
            "timezones": ["Гренада (UTC-04)"],
            "alpha-2-code": "GD",
            "alpha-3-code": "GRD",
            "continent": "Северная Америка",
            "name": "Гренада",
            "capital": "Сент-Джорджес",
        },
        {
            "timezones": ["Казахстан (UTC+05)", "Казахстан (UTC+06)"],
            "alpha-2-code": "KZ",
            "alpha-3-code": "KAZ",
            "continent": "Азия",
            "name": "Казахстан",
            "capital": "Нур-Султан (Астана)",
        },
        {
            "timezones": ["Лаос (UTC+07)"],
            "alpha-2-code": "LA",
            "alpha-3-code": "LAO",
            "continent": "Азия",
            "name": "Лаос",
            "capital": "Вьентьян",
        },
        {
            "timezones": ["Микронезия (UTC+10)", "Микронезия (UTC+11)"],
            "alpha-2-code": "FM",
            "alpha-3-code": "FSM",
            "continent": "Океания",
            "name": "Микронезия",
            "capital": "Паликир",
        },
        {
            "timezones": ["Молдавия (UTC+02)"],
            "alpha-2-code": "MD",
            "alpha-3-code": "MDA",
            "continent": "Европа",
            "name": "Молдавия",
            "capital": "Кишинев",
        },
        {
            "timezones": ["Монако (UTC+01)"],
            "alpha-2-code": "MC",
            "alpha-3-code": "MCO",
            "continent": "Европа",
            "name": "Монако",
            "capital": "Монако",
        },
        {
            "timezones": ["Черногория (UTC+01)"],
            "alpha-2-code": "ME",
            "alpha-3-code": "MNE",
            "continent": "Европа",
            "name": "Черногория",
            "capital": "Подгорица",
        },
        {
            "timezones": ["Марокко (UTC)"],
            "alpha-2-code": "MA",
            "alpha-3-code": "MAR",
            "continent": "Африка",
            "name": "Марокко",
            "capital": "Рабат",
        },
        {
            "timezones": ["Сент-Китс и Невис (UTC-04)"],
            "alpha-2-code": "KN",
            "alpha-3-code": "KNA",
            "continent": "Северная Америка",
            "name": "Сент-Китс и Невис",
            "capital": "Бастер",
        },
        {
            "timezones": ["Сент-Люсия (UTC-04)"],
            "alpha-2-code": "LC",
            "alpha-3-code": "LCA",
            "continent": "Северная Америка",
            "name": "Сент-Люсия",
            "capital": "Кастри",
        },
        {
            "timezones": ["Сент-Винсент и Гренадины (UTC-04)"],
            "alpha-2-code": "VC",
            "alpha-3-code": "VCT",
            "continent": "Северная Америка",
            "name": "Сент-Винсент и Гренадины",
            "capital": "Кингстаун",
        },
        {
            "timezones": ["Самоа (UTC+13)"],
            "alpha-2-code": "WS",
            "alpha-3-code": "WSM",
            "continent": "Океания",
            "name": "Самоа",
            "capital": "Апиа",
        },
        {
            "timezones": ["Сербия (UTC+01)"],
            "alpha-2-code": "RS",
            "alpha-3-code": "SRB",
            "continent": "Европа",
            "name": "Сербия",
            "capital": "Белград",
        },
        {
            "timezones": ["ЮАР (UTC+02)"],
            "alpha-2-code": "ZA",
            "alpha-3-code": "ZAF",
            "continent": "Африка",
            "name": "ЮАР",
            "capital": "Претория",
        },
        {
            "timezones": ["Испания (UTC)", "Испания (UTC+01)"],
            "alpha-2-code": "ES",
            "alpha-3-code": "ESP",
            "continent": "Европа",
            "name": "Испания",
            "capital": "Мадрид",
        },
        {
            "timezones": ["Шри-Ланка (UTC+05:30)"],
            "alpha-2-code": "LK",
            "alpha-3-code": "LKA",
            "continent": "Азия",
            "name": "Шри-Ланка",
            "capital": "Шри-Джаяварденепура-Котте",
        },
        {
            "timezones": ["Эсватини (Свазиленд) (UTC+02)"],
            "alpha-2-code": "SZ",
            "alpha-3-code": "SWZ",
            "continent": "Африка",
            "name": "Эсватини (Свазиленд)",
            "capital": "Мбабане",
        },
        {
            "timezones": ["Швейцария (UTC+01)"],
            "alpha-2-code": "CH",
            "alpha-3-code": "CHE",
            "continent": "Европа",
            "name": "Швейцария",
            "capital": "Берн",
        },
        {
            "timezones": ["ОАЭ (UTC+04)"],
            "alpha-2-code": "AE",
            "alpha-3-code": "ARE",
            "continent": "Азия",
            "name": "ОАЭ",
            "capital": "Абу-Даби",
        },
        {
            "timezones": ["Великобритания (UTC)"],
            "alpha-2-code": "GB",
            "alpha-3-code": "GBR",
            "continent": "Европа",
            "name": "Великобритания",
            "capital": "Лондон",
        },
    ]

    def day_of_week(self) -> str:
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self) -> str:
        month = self.month()
        return self.MONTH_NAMES[month]



import string

from collections import OrderedDict
from typing import Dict, Optional

from .. import BaseProvider, ElementsType


class Provider(BaseProvider):
    """Implement default file provider for Faker."""

    application_mime_types: ElementsType = (
        "application/atom+xml",  # Atom feeds
        "application/ecmascript",
        # ECMAScript/JavaScript; Defined in RFC 4329 (equivalent to
        # application/javascript but with stricter processing rules)
        "application/EDI-X12",  # EDI X12 data; Defined in RFC 1767
        "application/EDIFACT",  # EDI EDIFACT data; Defined in RFC 1767
        "application/json",  # JavaScript Object Notation JSON; Defined in RFC 4627
        # ECMAScript/JavaScript; Defined in RFC 4329 (equivalent to
        # application/ecmascript
        "application/javascript",
        #   but with looser processing rules) It is not accepted in IE 8
        #   or earlier - text/javascript is accepted but it is defined as obsolete in RFC 4329.
        #   The "type" attribute of the <script> tag in HTML5 is optional and in practice
        #   omitting the media type of JavaScript programs is the most interoperable
        #   solution since all browsers have always assumed the correct
        #   default even before HTML5.
        "application/octet-stream",
        # Arbitrary binary data.[6] Generally speaking this type identifies files that are not associated with
        # a specific application. Contrary to past assumptions by software packages such as Apache this is not
        # a type that should be applied to unknown files. In such a case, a server or application should not indicate
        # a content type, as it may be incorrect, but rather, should omit the type in order to allow the recipient
        # to guess the type.[7]
        "application/ogg",  # Ogg, a multimedia bitstream container format; Defined in RFC 5334
        "application/pdf",  # Portable Document Format, PDF has been in use for document exchange
        #   on the Internet since 1993; Defined in RFC 3778
        "application/postscript",  # PostScript; Defined in RFC 2046
        "application/rdf+xml",  # Resource Description Framework; Defined by RFC 3870
        "application/rss+xml",  # RSS feeds
        "application/soap+xml",  # SOAP; Defined by RFC 3902
        # Web Open Font Format; (candidate recommendation; use application/x-font-woff
        "application/font-woff",
        #   until standard is official)
        "application/xhtml+xml",  # XHTML; Defined by RFC 3236
        "application/xml-dtd",  # DTD files; Defined by RFC 3023
        "application/xop+xml",  # XOP
        "application/zip",  # ZIP archive files; Registered[8]
        "application/gzip",  # Gzip, Defined in RFC 6713
    )

    audio_mime_types: ElementsType = (
        "audio/basic",  # mulaw audio at 8 kHz, 1 channel; Defined in RFC 2046
        "audio/L24",  # 24bit Linear PCM audio at 8-48 kHz, 1-N channels; Defined in RFC 3190
        "audio/mp4",  # MP4 audio
        "audio/mpeg",  # MP3 or other MPEG audio; Defined in RFC 3003
        "audio/ogg",  # Ogg Vorbis, Speex, Flac and other audio; Defined in RFC 5334
        "audio/vorbis",  # Vorbis encoded audio; Defined in RFC 5215
        # RealAudio; Documented in RealPlayer Help[9]
        "audio/vnd.rn-realaudio",
        "audio/vnd.wave",  # WAV audio; Defined in RFC 2361
        "audio/webm",  # WebM open media format
    )

    image_mime_types: ElementsType = (
        "image/gif",  # GIF image; Defined in RFC 2045 and RFC 2046
        "image/jpeg",  # JPEG JFIF image; Defined in RFC 2045 and RFC 2046
        "image/pjpeg",
        # JPEG JFIF image; Associated with Internet Explorer; Listed in ms775147(v=vs.85) - Progressive JPEG,
        # initiated before global browser support for progressive JPEGs (Microsoft and Firefox).
        # Portable Network Graphics; Registered,[10] Defined in RFC 2083
        "image/png",
        "image/svg+xml",  # SVG vector image; Defined in SVG Tiny 1.2 Specification Appendix M
        # Tag Image File Format (only for Baseline TIFF); Defined in RFC 3302
        "image/tiff",
        "image/vnd.microsoft.icon",  # ICO image; Registered[11]
    )

    message_mime_types: ElementsType = (
        "message/http",  # Defined in RFC 2616
        "message/imdn+xml",  # IMDN Instant Message Disposition Notification; Defined in RFC 5438
        "message/partial",  # Email; Defined in RFC 2045 and RFC 2046
        # Email; EML files, MIME files, MHT files, MHTML files; Defined in RFC
        # 2045 and RFC 2046
        "message/rfc822",
    )

    model_mime_types: ElementsType = (
        "model/example",  # Defined in RFC 4735
        "model/iges",  # IGS files, IGES files; Defined in RFC 2077
        "model/mesh",  # MSH files, MESH files; Defined in RFC 2077, SILO files
        "model/vrml",  # WRL files, VRML files; Defined in RFC 2077
        # X3D ISO standard for representing 3D computer graphics, X3DB binary
        # files
        "model/x3d+binary",
        "model/x3d+vrml",  # X3D ISO standard for representing 3D computer graphics, X3DV VRML files
        "model/x3d+xml",  # X3D ISO standard for representing 3D computer graphics, X3D XML files
    )

    multipart_mime_types: ElementsType = (
        "multipart/mixed",  # MIME Email; Defined in RFC 2045 and RFC 2046
        "multipart/alternative",  # MIME Email; Defined in RFC 2045 and RFC 2046
        # MIME Email; Defined in RFC 2387 and used by MHTML (HTML mail)
        "multipart/related",
        "multipart/form-data",  # MIME Webform; Defined in RFC 2388
        "multipart/signed",  # Defined in RFC 1847
        "multipart/encrypted",  # Defined in RFC 1847
    )

    text_mime_types: ElementsType = (
        "text/cmd",  # commands; subtype resident in Gecko browsers like Firefox 3.5
        "text/css",  # Cascading Style Sheets; Defined in RFC 2318
        "text/csv",  # Comma-separated values; Defined in RFC 4180
        "text/html",  # HTML; Defined in RFC 2854
        "text/javascript",
        # (Obsolete): JavaScript; Defined in and obsoleted by RFC 4329 in order to discourage its usage in favor of
        # application/javascript. However, text/javascript is allowed in HTML 4 and 5 and, unlike
        # application/javascript, has cross-browser support. The "type" attribute of the <script> tag in HTML5 is
        # optional and there is no need to use it at all since all browsers have always assumed the correct default
        # (even in HTML 4 where it was required by the specification).
        "text/plain",  # Textual data; Defined in RFC 2046 and RFC 3676
        "text/vcard",  # vCard (contact information); Defined in RFC 6350
        "text/xml",  # Extensible Markup Language; Defined in RFC 3023
    )

    video_mime_types: ElementsType = (
        "video/mpeg",  # MPEG-1 video with multiplexed audio; Defined in RFC 2045 and RFC 2046
        "video/mp4",  # MP4 video; Defined in RFC 4337
        # Ogg Theora or other video (with audio); Defined in RFC 5334
        "video/ogg",
        "video/quicktime",  # QuickTime video; Registered[12]
        "video/webm",  # WebM Matroska-based open media format
        "video/x-matroska",  # Matroska open media format
        "video/x-ms-wmv",  # Windows Media Video; Documented in Microsoft KB 288102
        "video/x-flv",  # Flash video (FLV files)
    )

    mime_types: Dict[str, ElementsType] = OrderedDict(
        (
            ("application", application_mime_types),
            ("audio", audio_mime_types),
            ("image", image_mime_types),
            ("message", message_mime_types),
            ("model", model_mime_types),
            ("multipart", multipart_mime_types),
            ("text", text_mime_types),
            ("video", video_mime_types),
        )
    )

    audio_file_extensions: ElementsType = (
        "flac",
        "mp3",
        "wav",
    )

    image_file_extensions: ElementsType = (
        "bmp",
        "gif",
        "jpeg",
        "jpg",
        "png",
        "tiff",
    )

    text_file_extensions: ElementsType = (
        "css",
        "csv",
        "html",
        "js",
        "json",
        "txt",
    )

    video_file_extensions: ElementsType = (
        "mp4",
        "avi",
        "mov",
        "webm",
    )

    office_file_extensions: ElementsType = (
        "doc",  # legacy MS Word
        "docx",  # MS Word
        "xls",  # legacy MS Excel
        "xlsx",  # MS Excel
        "ppt",  # legacy MS PowerPoint
        "pptx",  # MS PowerPoint
        "odt",  # LibreOffice document
        "ods",  # LibreOffice spreadsheet
        "odp",  # LibreOffice presentation
        "pages",  # Apple Pages
        "numbers",  # Apple Numbers
        "key",  # Apple Keynote
        "pdf",  # Portable Document Format
    )

    file_extensions: Dict[str, ElementsType] = OrderedDict(
        (
            ("audio", audio_file_extensions),
            ("image", image_file_extensions),
            ("office", office_file_extensions),
            ("text", text_file_extensions),
            ("video", video_file_extensions),
        )
    )
    unix_device_prefixes: ElementsType = ("sd", "vd", "xvd")

    def mime_type(self, category: Optional[str] = None) -> str:
        """Generate a mime type under the specified ``category``.

        If ``category`` is ``None``, a random category will be used. The list of
        valid categories include ``'application'``, ``'audio'``, ``'image'``,
        ``'message'``, ``'model'``, ``'multipart'``, ``'text'``, and
        ``'video'``.

        :sample:
        :sample: category='application'
        """
        category = category if category else self.random_element(list(self.mime_types.keys()))
        return self.random_element(self.mime_types[category])

    def file_name(self, category: Optional[str] = None, extension: Optional[str] = None) -> str:
        """Generate a random file name with extension.

        If ``extension`` is ``None``, a random extension will be created under
        the hood using |file_extension| with the specified ``category``. If a
        value for ``extension`` is provided, the value will be used instead,
        and ``category`` will be ignored. The actual name part itself is
        generated using |word|.

        :sample size=10:
        :sample: category='audio'
        :sample: extension='abcdef'
        :sample: category='audio', extension='abcdef'
        """
        if extension is None:
            extension = self.file_extension(category)
        filename: str = self.generator.word()
        return f"{filename}.{extension}"

    def file_extension(self, category: Optional[str] = None) -> str:
        """Generate a file extension under the specified ``category``.

        If ``category`` is ``None``, a random category will be used. The list of
        valid categories include: ``'audio'``, ``'image'``, ``'office'``,
        ``'text'``, and ``'video'``.

        :sample:
        :sample: category='image'
        """
        if category is None:
            category = self.random_element(list(self.file_extensions.keys()))
        return self.random_element(self.file_extensions[category])

    def file_path(
        self,
        depth: int = 1,
        category: Optional[str] = None,
        extension: Optional[str] = None,
    ) -> str:
        """Generate an absolute pathname to a file.

        This method uses |file_name| under the hood to generate the file name
        itself, and ``depth`` controls the depth of the directory path, and
        |word| is used under the hood to generate the different directory names.

        :sample size=10:
        :sample: depth=3
        :sample: depth=5, category='video'
        :sample: depth=5, category='video', extension='abcdef'
        """
        file: str = self.file_name(category, extension)
        path: str = f"/{file}"
        for _ in range(0, depth):
            path = f"/{self.generator.word()}{path}"
        return path

    def unix_device(self, prefix: Optional[str] = None) -> str:
        """Generate a Unix device file name.

        If ``prefix`` is ``None``, a random prefix will be used. The list of
        valid prefixes include: ``'sd'``, ``'vd'``, and ``'xvd'``.

        :sample:
        :sample: prefix='mmcblk'
        """
        if prefix is None:
            prefix = self.random_element(self.unix_device_prefixes)
        suffix: str = self.random_element(string.ascii_lowercase)
        path = "/dev/%s%s" % (prefix, suffix)
        return path

    def unix_partition(self, prefix: Optional[str] = None) -> str:
        """Generate a Unix partition name.

        This method uses |unix_device| under the hood to create a device file
        name with the specified ``prefix``.

        :sample:
        :sample: prefix='mmcblk'
        """
        path: str = self.unix_device(prefix=prefix)
        path += str(self.random_digit())
        return path



from .. import Provider as InternetProvider


class Provider(InternetProvider):

    free_email_domains = (
        "gmail.com",
        "yahoo.com",
        "hotmail.com",
        "yahoo.com.au",
        "hotmail.com.au",
    )

    tlds = (
        "com",
        "com.au",
        "org",
        "org.au",
        "net",
        "net.au",
        "biz",
        "info",
        "edu",
        "edu.au",
    )



from datetime import date

from .. import Provider as SsnProvider


class Provider(SsnProvider):
    def ssn(self) -> str:
        """
        Ukrainian "Реєстраційний номер облікової картки платника податків"
        also known as "Ідентифікаційний номер фізичної особи".
        """
        digits = []

        # Number of days between 1899-12-31 and a birth date
        for digit in str((self.generator.date_object() - date(1899, 12, 31)).days):
            digits.append(int(digit))

        # Person's sequence number
        for _ in range(4):
            digits.append(self.random_int(0, 9))

        checksum = (
            digits[0] * -1
            + digits[1] * 5
            + digits[2] * 7
            + digits[3] * 9
            + digits[4] * 4
            + digits[5] * 6
            + digits[6] * 10
            + digits[7] * 5
            + digits[8] * 7
        )
        # Remainder of a checksum divided by 11 or 1 if it equals to 10
        digits.append(checksum % 11 % 10)

        return "".join(str(digit) for digit in digits)



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``he_IL`` locale."""

    """ Source : https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Israel  """
    license_formats = (
        "###-##-###",
        "##-###-##",
    )



from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    phonenumber_prefixes = [
        134,
        135,
        136,
        137,
        138,
        139,
        147,
        150,
        151,
        152,
        157,
        158,
        159,
        182,
        187,
        188,
        130,
        131,
        132,
        145,
        155,
        156,
        185,
        186,
        145,
        133,
        153,
        180,
        181,
        189,
    ]
    formats = [str(i) + "########" for i in phonenumber_prefixes]

    def phonenumber_prefix(self) -> int:
        return self.random_element(self.phonenumber_prefixes)



from .. import Provider as SsnProvider


class Provider(SsnProvider):
    ssn_formats = ("?#########",)

    def ssn(self) -> str:
        return self.bothify(self.random_element(self.ssn_formats)).upper()



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``en_NZ`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_New_Zealand
    """

    license_formats = (
        # Old plates
        "??%##",
        "??%###",
        "??%###",
        # Three letters since 2002
        "A??%##",
        "B??%##",
        "C??%##",
        "D??%##",
        "E??%##",
        "F??%##",
        "G??%##",
        "H??%##",
        "J??%##",
        "K??%##",
        "L??%##",
        "M??%##",
        # After 2018
        "N??%##",
    )



from typing import Tuple

from .. import Provider as AddressProvider


class Provider(AddressProvider):
    city_suffixes = (
        "do Sul",
        "do Norte",
        "de Minas",
        "do Campo",
        "Grande",
        "da Serra",
        "do Oeste",
        "de Goiás",
        "Paulista",
        "da Mata",
        "Alegre",
        "da Praia",
        "das Flores",
        "das Pedras",
        "dos Dourados",
        "do Amparo",
        "do Galho",
        "da Prata",
        "Verde",
    )
    street_prefixes = (
        "Aeroporto",
        "Alameda",
        "Área",
        "Avenida",
        "Campo",
        "Chácara",
        "Colônia",
        "Condomínio",
        "Conjunto",
        "Distrito",
        "Esplanada",
        "Estação",
        "Estrada",
        "Favela",
        "Fazenda",
        "Feira",
        "Jardim",
        "Ladeira",
        "Lago",
        "Lagoa",
        "Largo",
        "Loteamento",
        "Morro",
        "Núcleo",
        "Parque",
        "Passarela",
        "Pátio",
        "Praça",
        "Praia",
        "Quadra",
        "Recanto",
        "Residencial",
        "Rodovia",
        "Rua",
        "Setor",
        "Sítio",
        "Travessa",
        "Trecho",
        "Trevo",
        "Vale",
        "Vereda",
        "Via",
        "Viaduto",
        "Viela",
        "Vila",
    )
    city_formats = (
        "{{last_name}}",
        "{{last_name}}",
        "{{last_name}}",
        "{{last_name}}",
        "{{last_name}} {{city_suffix}}",
        "{{last_name}} {{city_suffix}}",
        "{{last_name}} {{city_suffix}}",
        "{{last_name}} de {{last_name}}",
    )
    street_name_formats = (
        "{{street_prefix}} {{last_name}}",
        "{{street_prefix}} {{first_name}} {{last_name}}",
        "{{street_prefix}} de {{last_name}}",
    )

    street_address_formats = (
        "{{street_name}}",
        "{{street_name}}, {{building_number}}",
        "{{street_name}}, {{building_number}}",
        "{{street_name}}, {{building_number}}",
        "{{street_name}}, {{building_number}}",
        "{{street_name}}, {{building_number}}",
        "{{street_name}}, {{building_number}}",
    )

    address_formats = ("{{street_address}}\n{{bairro}}\n{{postcode}} {{city}} / {{estado_sigla}}",)

    building_number_formats = ("%", "%#", "%#", "%#", "%##")

    postcode_raw_formats = ("########",)
    postcode_all_formats = postcode_raw_formats + ("#####-###",)

    bairros = (
        "Aarão Reis",
        "Acaba Mundo",
        "Acaiaca",
        "Ademar Maldonado",
        "Aeroporto",
        "Aguas Claras",
        "Alípio De Melo",
        "Alpes",
        "Alta Tensão 1ª Seção",
        "Alta Tensão 2ª Seção",
        "Alto Caiçaras",
        "Alto Das Antenas",
        "Alto Dos Pinheiros",
        "Alto Vera Cruz",
        "Álvaro Camargos",
        "Ambrosina",
        "Andiroba",
        "Antonio Ribeiro De Abreu 1ª Seção",
        "Aparecida 7ª Seção",
        "Ápia",
        "Apolonia",
        "Araguaia",
        "Atila De Paiva",
        "Bacurau",
        "Bairro Das Indústrias Ii",
        "Baleia",
        "Barão Homem De Melo 1ª Seção",
        "Barão Homem De Melo 2ª Seção",
        "Barão Homem De Melo 3ª Seção",
        "Barreiro",
        "Beija Flor",
        "Beira Linha",
        "Bela Vitoria",
        "Belmonte",
        "Bernadete",
        "Betânia",
        "Biquinhas",
        "Boa Esperança",
        "Boa União 1ª Seção",
        "Boa União 2ª Seção",
        "Boa Viagem",
        "Boa Vista",
        "Bom Jesus",
        "Bonfim",
        "Bonsucesso",
        "Brasil Industrial",
        "Braúnas",
        "Buraco Quente",
        "Cabana Do Pai Tomás",
        "Cachoeirinha",
        "Caetano Furquim",
        "Caiçara - Adelaide",
        "Calafate",
        "Califórnia",
        "Camargos",
        "Campo Alegre",
        "Camponesa 1ª Seção",
        "Camponesa 2ª Seção",
        "Canaa",
        "Canadá",
        "Candelaria",
        "Capitão Eduardo",
        "Cardoso",
        "Casa Branca",
        "Castanheira",
        "Cdi Jatoba",
        "Cenaculo",
        "Céu Azul",
        "Chácara Leonina",
        "Cidade Jardim Taquaril",
        "Cinquentenário",
        "Colégio Batista",
        "Comiteco",
        "Concórdia",
        "Cônego Pinheiro 1ª Seção",
        "Cônego Pinheiro 2ª Seção",
        "Confisco",
        "Conjunto Bonsucesso",
        "Conjunto Califórnia I",
        "Conjunto Califórnia Ii",
        "Conjunto Capitão Eduardo",
        "Conjunto Celso Machado",
        "Conjunto Floramar",
        "Conjunto Jardim Filadélfia",
        "Conjunto Jatoba",
        "Conjunto Lagoa",
        "Conjunto Minas Caixa",
        "Conjunto Novo Dom Bosco",
        "Conjunto Paulo Vi",
        "Conjunto Providencia",
        "Conjunto Santa Maria",
        "Conjunto São Francisco De Assis",
        "Conjunto Serra Verde",
        "Conjunto Taquaril",
        "Copacabana",
        "Coqueiros",
        "Corumbiara",
        "Custodinha",
        "Das Industrias I",
        "Delta",
        "Diamante",
        "Distrito Industrial Do Jatoba",
        "Dom Bosco",
        "Dom Cabral",
        "Dom Joaquim",
        "Dom Silverio",
        "Dona Clara",
        "Embaúbas",
        "Engenho Nogueira",
        "Ermelinda",
        "Ernesto Nascimento",
        "Esperança",
        "Estrela",
        "Estrela Do Oriente",
        "Etelvina Carneiro",
        "Europa",
        "Eymard",
        "Fazendinha",
        "Flamengo",
        "Flavio De Oliveira",
        "Flavio Marques Lisboa",
        "Floramar",
        "Frei Leopoldo",
        "Gameleira",
        "Garças",
        "Glória",
        "Goiania",
        "Graça",
        "Granja De Freitas",
        "Granja Werneck",
        "Grota",
        "Grotinha",
        "Guarani",
        "Guaratã",
        "Havaí",
        "Heliopolis",
        "Horto Florestal",
        "Inconfidência",
        "Indaiá",
        "Independência",
        "Ipe",
        "Itapoa",
        "Itatiaia",
        "Jaqueline",
        "Jaraguá",
        "Jardim Alvorada",
        "Jardim Atlântico",
        "Jardim Do Vale",
        "Jardim Dos Comerciarios",
        "Jardim Felicidade",
        "Jardim Guanabara",
        "Jardim Leblon",
        "Jardim Montanhês",
        "Jardim São José",
        "Jardim Vitoria",
        "Jardinópolis",
        "Jatobá",
        "João Alfredo",
        "João Paulo Ii",
        "Jonas Veiga",
        "Juliana",
        "Lagoa",
        "Lagoinha",
        "Lagoinha Leblon",
        "Lajedo",
        "Laranjeiras",
        "Leonina",
        "Leticia",
        "Liberdade",
        "Lindéia",
        "Lorena",
        "Madre Gertrudes",
        "Madri",
        "Mala E Cuia",
        "Manacas",
        "Mangueiras",
        "Mantiqueira",
        "Marajó",
        "Maravilha",
        "Marçola",
        "Maria Goretti",
        "Maria Helena",
        "Maria Tereza",
        "Maria Virgínia",
        "Mariano De Abreu",
        "Marieta 1ª Seção",
        "Marieta 2ª Seção",
        "Marieta 3ª Seção",
        "Marilandia",
        "Mariquinhas",
        "Marmiteiros",
        "Milionario",
        "Minas Brasil",
        "Minas Caixa",
        "Minaslandia",
        "Mineirão",
        "Miramar",
        "Mirante",
        "Mirtes",
        "Monsenhor Messias",
        "Monte Azul",
        "Monte São José",
        "Morro Dos Macacos",
        "Nazare",
        "Nossa Senhora Aparecida",
        "Nossa Senhora Da Aparecida",
        "Nossa Senhora Da Conceição",
        "Nossa Senhora De Fátima",
        "Nossa Senhora Do Rosário",
        "Nova America",
        "Nova Cachoeirinha",
        "Nova Cintra",
        "Nova Esperança",
        "Nova Floresta",
        "Nova Gameleira",
        "Nova Pampulha",
        "Novo Aarão Reis",
        "Novo Das Industrias",
        "Novo Glória",
        "Novo Santa Cecilia",
        "Novo Tupi",
        "Oeste",
        "Olaria",
        "Olhos D'água",
        "Ouro Minas",
        "Pantanal",
        "Paquetá",
        "Paraíso",
        "Parque São José",
        "Parque São Pedro",
        "Paulo Vi",
        "Pedreira Padro Lopes",
        "Penha",
        "Petropolis",
        "Pilar",
        "Pindorama",
        "Pindura Saia",
        "Piraja",
        "Piratininga",
        "Pirineus",
        "Pompéia",
        "Pongelupe",
        "Pousada Santo Antonio",
        "Primeiro De Maio",
        "Providencia",
        "Ribeiro De Abreu",
        "Rio Branco",
        "Salgado Filho",
        "Santa Amelia",
        "Santa Branca",
        "Santa Cecilia",
        "Santa Cruz",
        "Santa Helena",
        "Santa Inês",
        "Santa Isabel",
        "Santa Margarida",
        "Santa Maria",
        "Santa Rita",
        "Santa Rita De Cássia",
        "Santa Sofia",
        "Santa Terezinha",
        "Santana Do Cafezal",
        "Santo André",
        "São Benedito",
        "São Bernardo",
        "São Cristóvão",
        "São Damião",
        "São Francisco",
        "São Francisco Das Chagas",
        "São Gabriel",
        "São Geraldo",
        "São Gonçalo",
        "São João",
        "São João Batista",
        "São Jorge 1ª Seção",
        "São Jorge 2ª Seção",
        "São Jorge 3ª Seção",
        "São José",
        "São Marcos",
        "São Paulo",
        "São Salvador",
        "São Sebastião",
        "São Tomaz",
        "São Vicente",
        "Satelite",
        "Saudade",
        "Senhor Dos Passos",
        "Serra Do Curral",
        "Serra Verde",
        "Serrano",
        "Solar Do Barreiro",
        "Solimoes",
        "Sport Club",
        "Suzana",
        "Taquaril",
        "Teixeira Dias",
        "Tiradentes",
        "Tirol",
        "Tres Marias",
        "Trevo",
        "Túnel De Ibirité",
        "Tupi A",
        "Tupi B",
        "União",
        "Unidas",
        "Universitário",
        "Universo",
        "Urca",
        "Vale Do Jatoba",
        "Varzea Da Palma",
        "Venda Nova",
        "Ventosa",
        "Vera Cruz",
        "Vila Aeroporto",
        "Vila Aeroporto Jaraguá",
        "Vila Antena",
        "Vila Antena Montanhês",
        "Vila Atila De Paiva",
        "Vila Bandeirantes",
        "Vila Barragem Santa Lúcia",
        "Vila Batik",
        "Vila Betânia",
        "Vila Boa Vista",
        "Vila Calafate",
        "Vila Califórnia",
        "Vila Canto Do Sabiá",
        "Vila Cemig",
        "Vila Cloris",
        "Vila Copacabana",
        "Vila Copasa",
        "Vila Coqueiral",
        "Vila Da Amizade",
        "Vila Da Ária",
        "Vila Da Luz",
        "Vila Da Paz",
        "Vila Das Oliveiras",
        "Vila Do Pombal",
        "Vila Dos Anjos",
        "Vila Ecológica",
        "Vila Engenho Nogueira",
        "Vila Esplanada",
        "Vila Formosa",
        "Vila Fumec",
        "Vila Havaí",
        "Vila Independencia 1ª Seção",
        "Vila Independencia 2ª Seção",
        "Vila Independencia 3ª Seção",
        "Vila Inestan",
        "Vila Ipiranga",
        "Vila Jardim Alvorada",
        "Vila Jardim Leblon",
        "Vila Jardim São José",
        "Vila Madre Gertrudes 1ª Seção",
        "Vila Madre Gertrudes 2ª Seção",
        "Vila Madre Gertrudes 3ª Seção",
        "Vila Madre Gertrudes 4ª Seção",
        "Vila Maloca",
        "Vila Mangueiras",
        "Vila Mantiqueira",
        "Vila Maria",
        "Vila Minaslandia",
        "Vila Nossa Senhora Do Rosário",
        "Vila Nova",
        "Vila Nova Cachoeirinha 1ª Seção",
        "Vila Nova Cachoeirinha 2ª Seção",
        "Vila Nova Cachoeirinha 3ª Seção",
        "Vila Nova Dos Milionarios",
        "Vila Nova Gameleira 1ª Seção",
        "Vila Nova Gameleira 2ª Seção",
        "Vila Nova Gameleira 3ª Seção",
        "Vila Nova Paraíso",
        "Vila Novo São Lucas",
        "Vila Oeste",
        "Vila Olhos D'água",
        "Vila Ouro Minas",
        "Vila Paquetá",
        "Vila Paraíso",
        "Vila Petropolis",
        "Vila Pilar",
        "Vila Pinho",
        "Vila Piratininga",
        "Vila Piratininga Venda Nova",
        "Vila Primeiro De Maio",
        "Vila Puc",
        "Vila Real 1ª Seção",
        "Vila Real 2ª Seção",
        "Vila Rica",
        "Vila Santa Monica 1ª Seção",
        "Vila Santa Monica 2ª Seção",
        "Vila Santa Rosa",
        "Vila Santo Antônio",
        "Vila Santo Antônio Barroquinha",
        "Vila São Dimas",
        "Vila São Francisco",
        "Vila São Gabriel",
        "Vila São Gabriel Jacui",
        "Vila São Geraldo",
        "Vila São João Batista",
        "Vila São Paulo",
        "Vila São Rafael",
        "Vila Satélite",
        "Vila Sesc",
        "Vila Sumaré",
        "Vila Suzana Primeira Seção",
        "Vila Suzana Segunda Seção",
        "Vila Tirol",
        "Vila Trinta E Um De Março",
        "Vila União",
        "Vila Vista Alegre",
        "Virgínia",
        "Vista Alegre",
        "Vista Do Sol",
        "Vitoria",
        "Vitoria Da Conquista",
        "Xangri-Lá",
        "Xodo-Marize",
        "Zilah Sposito",
        "Outro",
        "Novo São Lucas",
        "Esplanada",
        "Estoril",
        "Novo Ouro Preto",
        "Ouro Preto",
        "Padre Eustáquio",
        "Palmares",
        "Palmeiras",
        "Vila De Sá",
        "Floresta",
        "Anchieta",
        "Aparecida",
        "Grajaú",
        "Planalto",
        "Bandeirantes",
        "Gutierrez",
        "Jardim América",
        "Renascença",
        "Barro Preto",
        "Barroca",
        "Sagrada Família",
        "Ipiranga",
        "Belvedere",
        "Santa Efigênia",
        "Santa Lúcia",
        "Santa Monica",
        "Vila Jardim Montanhes",
        "Santa Rosa",
        "Santa Tereza",
        "Buritis",
        "Vila Paris",
        "Santo Agostinho",
        "Santo Antônio",
        "Caiçaras",
        "São Bento",
        "Prado",
        "Lourdes",
        "Fernão Dias",
        "Carlos Prates",
        "Carmo",
        "Luxemburgo",
        "São Lucas",
        "São Luiz",
        "Mangabeiras",
        "São Pedro",
        "Horto",
        "Cidade Jardim",
        "Castelo",
        "Cidade Nova",
        "Savassi",
        "Serra",
        "Silveira",
        "Sion",
        "Centro",
        "Alto Barroca",
        "Nova Vista",
        "Coração De Jesus",
        "Coração Eucarístico",
        "Funcionários",
        "Cruzeiro",
        "João Pinheiro",
        "Nova Granada",
        "Nova Suíça",
        "Itaipu",
    )
    countries = (
        "Afeganistão",
        "África do Sul",
        "Akrotiri",
        "Albânia",
        "Alemanha",
        "Andorra",
        "Angola",
        "Anguila",
        "Antártica",
        "Antígua e Barbuda",
        "Antilhas Holandesas",
        "Arábia Saudita",
        "Argélia",
        "Argentina",
        "Armênia",
        "Aruba",
        "Ashmore and Cartier Islands",
        "Austrália",
        "Áustria",
        "Azerbaijão",
        "Bahamas",
        "Bangladesh",
        "Barbados",
        "Barein",
        "Bélgica",
        "Belize",
        "Benim",
        "Bermudas",
        "Bielorrússia",
        "Birmânia",
        "Bolívia",
        "Bósnia e Herzegovina",
        "Botsuana",
        "Brasil",
        "Brunei",
        "Bulgária",
        "Burquina Faso",
        "Burundi",
        "Butão",
        "Cabo Verde",
        "Camarões",
        "Camboja",
        "Canadá",
        "Catar",
        "Cazaquistão",
        "Chade",
        "Chile",
        "China",
        "Chipre",
        "Clipperton Island",
        "Colômbia",
        "Comores",
        "Congo-Brazzaville",
        "Congo-Kinshasa",
        "Coral Sea Islands",
        "Coreia do Norte",
        "Coreia do Sul",
        "Costa do Marfim",
        "Costa Rica",
        "Croácia",
        "Cuba",
        "Dhekelia",
        "Dinamarca",
        "Domínica",
        "Egito",
        "Costa do Marfim",
        "Costa Rica",
        "Croácia",
        "Cuba",
        "Dhekelia",
        "Dinamarca",
        "Domínica",
        "Egito",
        "Emirados Árabes Unidos",
        "Equador",
        "Eritreia",
        "Eslováquia",
        "Eslovênia",
        "Espanha",
        "Estados Unidos",
        "Estônia",
        "Etiópia",
        "Faroé",
        "Fiji",
        "Filipinas",
        "Finlândia",
        "França",
        "Gabão",
        "Gâmbia",
        "Gana",
        "Geórgia",
        "Geórgia do Sul e Sandwich do Sul",
        "Gibraltar",
        "Granada",
        "Grécia",
        "Gronelândia",
        "Guam",
        "Guatemala",
        "Guernsey",
        "Guiana",
        "Guiné",
        "Guiné Equatorial",
        "Guiné-Bissau",
        "Haiti",
        "Honduras",
        "Hong Kong",
        "Hungria",
        "Iêmen",
        "Ilha Bouvet",
        "Ilha do Natal",
        "Ilha Norfolk",
        "Ilhas Caiman",
        "Ilhas Cook",
        "Ilhas dos Cocos",
        "Ilhas Falkland",
        "Ilhas Heard e McDonald",
        "Ilhas Marshall",
        "Ilhas Salomão",
        "Ilhas Turcas e Caicos",
        "Ilhas Virgens Americanas",
        "Ilhas Virgens Britânicas",
        "Índia",
        "Indonésia",
        "Iran",
        "Iraque",
        "Irlanda",
        "Islândia",
        "Israel",
        "Itália",
        "Jamaica",
        "Jan Mayen",
        "Japão",
        "Jersey",
        "Jibuti",
        "Jordânia",
        "Kuwait",
        "Laos",
        "Lesoto",
        "Letônia",
        "Líbano",
        "Libéria",
        "Líbia",
        "Liechtenstein",
        "Lituânia",
        "Luxemburgo",
        "Macau",
        "Macedônia",
        "Madagáscar",
        "Malásia",
        "Malávi",
        "Maldivas",
        "Mali",
        "Malta",
        "Man, Isle of",
        "Marianas do Norte",
        "Marrocos",
        "Maurícia",
        "Mauritânia",
        "Mayotte",
        "México",
        "Micronésia",
        "Moçambique",
        "Moldávia",
        "Mônaco",
        "Mongólia",
        "Monserrate",
        "Montenegro",
        "Namíbia",
        "Nauru",
        "Navassa Island",
        "Nepal",
        "Nicarágua",
        "Níger",
        "Nigéria",
        "Niue",
        "Noruega",
        "Nova Caledónia",
        "Nova Zelândia",
        "Omã",
        "Países Baixos",
        "Palau",
        "Panamá",
        "Papua-Nova Guiné",
        "Paquistão",
        "Paracel Islands",
        "Paraguai",
        "Peru",
        "Pitcairn",
        "Polinésia Francesa",
        "Polônia",
        "Porto Rico",
        "Portugal",
        "Quênia",
        "Quirguizistão",
        "Quiribáti",
        "Reino Unido",
        "República Centro-Africana",
        "República Checa",
        "República Dominicana",
        "Roménia",
        "Ruanda",
        "Rússia",
        "Salvador",
        "Samoa",
        "Samoa Americana",
        "Santa Helena",
        "Santa Lúcia",
        "São Cristóvão e Neves",
        "São Marinho",
        "São Pedro e Miquelon",
        "São Tomé e Príncipe",
        "São Vicente e Granadinas",
        "Sara Ocidental",
        "Seicheles",
        "Senegal",
        "Serra Leoa",
        "Sérvia",
        "Singapura",
        "Síria",
        "Somália",
        "Sri Lanka",
        "Suazilândia",
        "Sudão",
        "Suécia",
        "Suíça",
        "Suriname",
        "Svalbard e Jan Mayen",
        "Tailândia",
        "Taiwan",
        "Tajiquistão",
        "Tanzânia",
        "Território Britânico do Oceano Índico",
        "Territórios Austrais Franceses",
        "Timor Leste",
        "Togo",
        "Tokelau",
        "Tonga",
        "Trindade e Tobago",
        "Tunísia",
        "Turquemenistão",
        "Turquia",
        "Tuvalu",
        "Ucrânia",
        "Uganda",
        "União Europeia",
        "Uruguai",
        "Usbequistão",
        "Vanuatu",
        "Vaticano",
        "Venezuela",
        "Vietnam",
        "Wake Island",
        "Wallis e Futuna",
        "Zâmbia",
        "Zimbabué",
    )

    estados = (
        ("AC", "Acre"),
        ("AL", "Alagoas"),
        ("AP", "Amapá"),
        ("AM", "Amazonas"),
        ("BA", "Bahia"),
        ("CE", "Ceará"),
        ("DF", "Distrito Federal"),
        ("ES", "Espírito Santo"),
        ("GO", "Goiás"),
        ("MA", "Maranhão"),
        ("MT", "Mato Grosso"),
        ("MS", "Mato Grosso do Sul"),
        ("MG", "Minas Gerais"),
        ("PA", "Pará"),
        ("PB", "Paraíba"),
        ("PR", "Paraná"),
        ("PE", "Pernambuco"),
        ("PI", "Piauí"),
        ("RJ", "Rio de Janeiro"),
        ("RN", "Rio Grande do Norte"),
        ("RS", "Rio Grande do Sul"),
        ("RO", "Rondônia"),
        ("RR", "Roraima"),
        ("SC", "Santa Catarina"),
        ("SP", "São Paulo"),
        ("SE", "Sergipe"),
        ("TO", "Tocantins"),
    )

    def street_prefix(self) -> str:
        """
        :example 'rua'
        """
        return self.random_element(self.street_prefixes)

    def estado(self) -> Tuple[str, str]:
        """
        Randomly returns a Brazilian State  ('sigla' , 'nome').
        :example ('MG' . 'Minas Gerais')
        """
        return self.random_element(self.estados)

    def estado_nome(self) -> str:
        """
        Randomly returns a Brazilian State Name
        :example 'Minas Gerais'
        """
        return self.estado()[1]

    def estado_sigla(self) -> str:
        """
        Randomly returns the abbreviation of a Brazilian State

        :example 'MG'
        """
        return self.estado()[0]

    def bairro(self) -> str:
        """
        Randomly returns a bairro (neighborhood) name.
        The names were taken from the city of Belo Horizonte - Minas Gerais

        :example 'Serra'
        """
        return self.random_element(self.bairros)

    def postcode(self, formatted: bool = True) -> str:
        """
        Randomly returns a postcode.
        :param formatted: True to allow formatted postcodes, else False (default True)
        :example formatted: '41224-212' '83992-291' '12324322'
        :example raw: '43920231' '34239530'
        """
        template = self.postcode_all_formats if formatted else self.postcode_raw_formats
        return self.bothify(self.random_element(template))

    # aliases
    def neighborhood(self) -> str:
        return self.bairro()

    def administrative_unit(self) -> str:
        return self.estado_nome()

    state = administrative_unit

    def state_abbr(self) -> str:
        return self.estado_sigla()
